@@ -12,9 +12,13 @@
 #include <vscreen/vs_togglebutton.h>
 #include <vscreen/vs_editline.h>
 
+#include <sigc++/object.h>
+
 // This basically just provides a generic interface to commit changes to apt
-// options.
-class apt_config_widget:public SigC::Object
+// options.  Not a descendant of Object because the classes below get Object
+// via vscreen_widget, and things get confused if Object is included twice.
+// (the compiler gets unhappy)
+class apt_config_widget
 {
 public:
   virtual ~apt_config_widget() {}
