@@ -18,18 +18,24 @@ download_manager *gen_cmdline_download_progress();
 int cmdline_update(int argc, char *argv[]);
 int cmdline_clean(int argc, char *argv[], bool simulate);
 int cmdline_autoclean(int argc, char *argv[], bool simulate);
-int cmdline_forget_new(int argc, char *argv[], bool simulate);
+int cmdline_forget_new(int argc, char *argv[],
+		       const char *status_fname, bool simulate);
 int cmdline_search(int argc, char *argv[],
-		   std::string display_format, std::string width);
-int cmdline_pkgaction(int argc, char *argv[], bool simulate,
+		   const char *status_fname,
+		   std::string display_format, std::string width,
+		   std::string sort);
+int cmdline_pkgaction(int argc, char *argv[],
+		      const char *status_fname, bool simulate,
 		      bool assume_yes, bool download_only, bool fix_broken,
-		      bool showvers, bool showdeps, bool always_prompt,
-		      int verbose);
-int cmdline_upgrade(int argc, char *argv[], bool simulate,
+		      bool showvers, bool showdeps, bool showsize,
+		      bool always_prompt, int verbose);
+int cmdline_upgrade(int argc, char *argv[],
+		    const char *status_fname, bool simulate,
 		    bool assume_yes, bool fix_broken,
-		    bool showvers, bool showdeps, bool always_prompt,
-		    int verbose);
+		    bool showvers, bool showdeps, bool showsize,
+		    bool always_prompt, int verbose);
 int cmdline_download(int argc, char *argv[]);
 int cmdline_moo(int argc, char *argv[], int verbose);
+int cmdline_show(int argc, char *argv[], int verbose);
 
 #endif
