@@ -33,11 +33,16 @@
 #include <apt-pkg/version.h>
 
 using namespace std;
+using namespace __gnu_cxx;
 
 class pkg_depitem:public pkg_subtree
 {
   pkgCache::DepIterator firstdep;
 
+  // calculated at initialization: true iff at least one package
+  // fulfilling the dependency is available (not necessarily installable)
+  bool available;
+
   bool is_broken();
   // Returns true if the dependency should be displayed as a broken dependency
 public:
@@ -53,7 +58,7 @@ public:
 };
 
 pkg_depitem::pkg_depitem(pkgCache::DepIterator &first, pkg_signal *sig):
-  pkg_subtree(""),firstdep(first)
+  pkg_subtree(""),firstdep(first),available(false)
 {
   pkgCache::DepIterator start,end;
   first.GlobOr(start,end);
@@ -80,22 +85,18 @@ pkg_depitem::pkg_depitem(pkgCache::DepIterator &first, pkg_signal *sig):
 	}
 
       for(pkgCache::VerIterator i=start.TargetPkg().VersionList(); !i.end(); i++)
-#ifndef HAVE_LIBAPT_PKG3
-	if(pkgCheckDep(start.TargetVer(), i.VerStr(), start->CompareOp))
-#else
 	if(_system->VS->CheckDep(i.VerStr(), start->CompareOp, start.TargetVer()))
-	  //if(1)
-#endif
-	  add_child(new pkg_ver_item(i, sig, is_or));
+	  {
+	    available=true;
+	    add_child(new pkg_ver_item(i, sig, is_or));
+	  }
 
       for(pkgCache::PrvIterator i=start.TargetPkg().ProvidesList(); !i.end(); i++)
-#ifndef HAVE_LIBAPT_PKG3
-	if(pkgCheckDep(start.TargetVer(), i.ProvideVersion(), start->CompareOp))
-#else
 	if(_system->VS->CheckDep(i.ProvideVersion(), start->CompareOp, start.TargetVer()))
-	  //if(1)
-#endif
-	  add_child(new pkg_ver_item(i.OwnerVer(), sig, true));
+	  {
+	    available=true;
+	    add_child(new pkg_ver_item(i.OwnerVer(), sig, true));
+	  }
 
       if(start==end)
 	break;
@@ -128,8 +129,16 @@ bool pkg_depitem::is_broken()
 void pkg_depitem::paint(vs_tree *win, int y, bool hierarchical)
 {
   if(is_broken())
-    vs_subtree<pkg_tree_node>::paint(win, y, hierarchical,
-				     get_name()+_(" (UNSATISFIED)"));
+    {
+      // If we can quickly prove that this is not satisfiable, tell
+      // the user that.  (specifically, if there are no available packages
+      // fulfilling the dependency)
+
+      const char *broken_str=available?_(" (UNSATISFIED)"):_(" (UNAVAILABLE)");
+
+      vs_subtree<pkg_tree_node>::paint(win, y, hierarchical,
+				       get_name()+broken_str);
+    }
   else
     pkg_subtree::paint(win, y, hierarchical);
 }
@@ -235,11 +244,7 @@ void setup_package_deps(const pkgCache::PkgIterator &pkg,
 	{
 	  if((ver.end() && (D->CompareOp&0x0F)!=pkgCache::Dep::NoOp) ||
 	     (!ver.end() &&
-#ifndef HAVE_LIBAPT_PKG3
-	      !pkgCheckDep(D.TargetVer(), ver.VerStr(), D->CompareOp)))
-#else
 	      !_system->VS->CheckDep(ver.VerStr(), D->CompareOp, D.TargetVer())))
-#endif
 
 	    {
 	      D++;
@@ -275,11 +280,7 @@ void setup_package_deps(const pkgCache::PkgIterator &pkg,
       {
 	for(pkgCache::DepIterator D=i.ParentPkg().RevDependsList(); !D.end(); D++)
 	  {
-#ifndef HAVE_LIBAPT_PKG3
-	    if(pkgCheckDep(D.TargetVer(), i.ProvideVersion(), D->CompareOp))
-#else
 	    if(_system->VS->CheckDep(i.ProvideVersion(), D->CompareOp, D.TargetVer()))
-#endif
 	      {
 		pkg_subtree *subtree;
 		tree_map::iterator found=subtrees.find(D.DepType());
