@@ -46,7 +46,7 @@
 class download_item;
 
 #if defined(HAVE_HASH_MAP) || defined(HAVE_EXT_HASH_MAP)
-namespace std {
+namespace HASH_NAMESPACE {
   struct hash<void *>
   {
     size_t operator()(void * __x) const { return (size_t) __x; }
@@ -67,7 +67,7 @@ public:
 
 class download_screen:public vs_tree, public pkgAcquireStatus
 {
-  typedef std::hash_map<void *, download_item *> downloadmap;
+  typedef HASH_NAMESPACE::hash_map<void *, download_item *> downloadmap;
   downloadmap active_items;
   // Makes it easy to find a currently downloading item when we get a hit
   // for it.
