@@ -33,17 +33,20 @@
 #include <apt-pkg/tagfile.h>
 #include <apt-pkg/fileutl.h>
 #include <apt-pkg/algorithms.h>
-#ifdef HAVE_LIBAPT_PKG3
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/policy.h>
 #include <apt-pkg/version.h>
-#endif
 
 #include <vector>
 
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
+#include <assert.h>
+
+#include <sigc++/bind.h>
+#include <sigc++/hide.h>
+#include <sigc++/object_slot.h>
 
 using namespace std;
 
@@ -62,18 +65,26 @@ class aptitudeDepCache::apt_undoer:public undoable
   int prev_flags; /* For Delete, tracks Purge mode; for Install, tracks 
 		   * Reinstall mode..
 		   */
-  changed_reason prev_installremovereason;
+  bool prev_autoinstalled;
+  changed_reason prev_installreason, prev_removereason;
   pkgCache::State::PkgSelectedState prev_selection_state;
 
+  string prev_forbidver;
+
   aptitudeDepCache *owner;
 public:
   apt_undoer(PkgIterator _pkg, int _prev_mode, int _prev_flags,
-	     changed_reason _prev_installremovereason,
+	     changed_reason _prev_installreason,
+	     changed_reason _prev_removereason,
 	     pkgCache::State::PkgSelectedState _prev_selection_state,
+	     string _prev_forbidver,
 	     aptitudeDepCache *_owner)
     :pkg(_pkg), prev_mode(_prev_mode), prev_flags(_prev_flags),
-     prev_installremovereason(_prev_installremovereason),
-     prev_selection_state(_prev_selection_state), owner(_owner)
+     prev_installreason(_prev_installreason),
+     prev_removereason(_prev_removereason),
+     prev_selection_state(_prev_selection_state),
+     prev_forbidver(_prev_forbidver),
+     owner(_owner)
   {
   }
 
@@ -84,7 +95,8 @@ public:
     else switch(prev_mode)
       {
       case ModeDelete:
-	owner->internal_mark_delete(pkg, prev_flags&Purge, prev_installremovereason==unused, NULL, false);
+	// the unused_delete parameter isn't that important..
+	owner->internal_mark_delete(pkg, prev_flags&Purge, prev_removereason==unused, NULL, false);
 	break;
       case ModeKeep:
 	owner->internal_mark_keep(pkg, prev_flags&AutoKept, prev_selection_state==pkgCache::State::Hold, NULL, false);
@@ -94,7 +106,10 @@ public:
 	break;
       }
 
-    owner->get_ext_state(pkg).installremove_reason=prev_installremovereason;
+    // make sure that everything is really set.
+    owner->get_ext_state(pkg).install_reason=prev_installreason;
+    owner->get_ext_state(pkg).remove_reason=prev_removereason;
+    owner->get_ext_state(pkg).forbidver=prev_forbidver;
   }
 };
 
@@ -129,7 +144,6 @@ public:
   }
 };
 
-#ifdef HAVE_LIBAPT_PKG3
 class aptitudeDepCache::candver_undoer:public undoable
 // Undoes a "set candidate version" command.
 {
@@ -148,31 +162,29 @@ public:
     owner->set_candidate_version(oldver, NULL);
   }
 };
-#endif
 
-#ifndef HAVE_LIBAPT_PKG3
-aptitudeDepCache::aptitudeDepCache(MMap &Map, OpProgress &Prog,
-				   bool do_initselections,
-				   bool WithLock,
-				   const char *status_fname)
-  :pkgDepCache(Map, Prog), dirty(false), package_states(NULL), lock(-1),
-   group_level(0), mark_and_sweep_in_progress(false)
-{
-  if(!_error->PendingError())
-    build_selection_list(Prog, WithLock, do_initselections, status_fname);
-}
-#else
 aptitudeDepCache::aptitudeDepCache(pkgCache *Cache, Policy *Plcy)
   :pkgDepCache(Cache, Plcy), dirty(false), package_states(NULL), lock(-1),
    group_level(0), mark_and_sweep_in_progress(false)
 {
+  // When the "install recommended packages" flag changes, collect garbage.
+  //
+  // (oh god make the indentation pain stop)
+  aptcfg->connectB(PACKAGE "::Recommends-Important",
+		   SigC::hide<bool>(SigC::bind<undo_group*, void, undo_group*>(SigC::slot(*this,
+											  &aptitudeDepCache::mark_and_sweep),
+									       NULL)));
+
+  aptcfg->connectB(PACKAGE "::Suggests-Important",
+		   SigC::hide<bool>(SigC::bind<undo_group*, void, undo_group*>(SigC::slot(*this,
+											  &aptitudeDepCache::mark_and_sweep),
+									       NULL)));
 }
 
 bool aptitudeDepCache::Init(OpProgress *Prog, bool WithLock, bool do_initselections, const char *status_fname)
 {
   return build_selection_list(*Prog, WithLock, do_initselections, status_fname);
 }
-#endif
 
 aptitudeDepCache::~aptitudeDepCache()
 {
@@ -200,7 +212,8 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
       package_states[i].new_package=true;
       package_states[i].selection_state=package_states[i].dselect_state=pkgCache::State::Unknown;
       package_states[i].reinstall=false;
-      package_states[i].installremove_reason=manual;
+      package_states[i].install_reason=manual;
+      package_states[i].remove_reason=manual;
     }
 
   if(WithLock && lock==-1)
@@ -240,11 +253,8 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
       int file_size=state_file.Size();
       Prog.OverallProgress(0, file_size, 1, _("Reading extended state information"));
 
-#ifndef HAVE_LIBAPT_PKG3
-      pkgTagFile tagfile(state_file);
-#else
+      begin_action_group();
       pkgTagFile tagfile(&state_file);
-#endif
       pkgTagSection section;
       int amt=0;
       bool do_dselect=aptcfg->FindB(PACKAGE "::Track-Dselect-State", true);
@@ -264,16 +274,25 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	      pkg_state.new_package=(tmp==1);
 
 	      tmp=0;
-	      section.FindFlag("ReInstall", tmp, 1);
-	      pkg_state.reinstall=(tmp==1);
+	      section.FindFlag("Upgrade", tmp, 1);
+	      pkg_state.upgrade=(tmp==1);
 
-	      pkg_state.installremove_reason=(changed_reason) section.FindI("Last-Change", manual);
+	      // The install reason is much more important to preserve
+	      // from previous versions, so support the outdated name
+	      // for it.
+	      pkg_state.install_reason=(changed_reason)
+		section.FindI("Install-Reason",
+			      section.FindI("Last-Change", manual));
+
+	      pkg_state.remove_reason=(changed_reason)
+		section.FindI("Remove-Reason", manual);
 
 	      candver=section.FindS("Version");
 
 	      pkg_state.selection_state=(pkgCache::State::PkgSelectedState) section.FindI("State", pkgCache::State::Unknown);
 	      pkg_state.dselect_state=(pkgCache::State::PkgSelectedState) section.FindI("Dselect-State", pkg->SelectedState);
 	      pkg_state.candver=candver;
+	      pkg_state.forbidver=section.FindS("ForbidVer");
 
 	      if(do_dselect && pkg->SelectedState!=pkg_state.dselect_state &&
 		 do_initselections)
@@ -285,9 +304,14 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	  amt+=section.size();
 	  Prog.OverallProgress(amt, file_size, 1, _("Reading extended state information"));
 	}
-      Prog.OverallProgress(amt, file_size, 1, _("Reading extended state information"));
+      Prog.OverallProgress(file_size, file_size, 1, _("Reading extended state information"));
+      end_action_group(NULL);
     }
 
+  int num=0;
+
+  Prog.OverallProgress(0, Head().PackageCount, 1, _("Initializing package states"));
+
   // Act on them
   for(pkgCache::PkgIterator i=PkgBegin(); !i.end(); i++)
     {
@@ -311,7 +335,6 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 
 	  // FIXME: should I check this for "unknown" packages as well?
 	  // Does that even make sense??
-#ifdef HAVE_LIBAPT_PKG3
 	  if(!estate.candver.empty() &&
 	     estate.candver!=state.InstVerIter(GetCache()).VerStr())
 	    {
@@ -324,11 +347,15 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 		  }
 	    }
 	  else
-#endif
 	    if(i.CurrentVer().end())
 	      MarkInstall(i);
 	    else
-	      SetReInstall(i, estate.reinstall);
+	      {
+		SetReInstall(i, estate.reinstall);
+
+		if(estate.upgrade && state.Upgradable())
+		  MarkInstall(i);
+	      }
 	  break;
 	case pkgCache::State::Hold:
 	  if(!do_initselections)
@@ -351,16 +378,23 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	    MarkDelete(i, true);
 	  break;
 	}
+
+      ++num;
+      Prog.OverallProgress(num, Head().PackageCount, 1, _("Initializing package states"));
     }
 
+  Prog.OverallProgress(Head().PackageCount, Head().PackageCount, 1, _("Initializing package states"));
+
   duplicate_cache(&backup_state);
 
-  if(aptcfg->FindB(PACKAGE "::Auto-Upgrade", true) && do_initselections)
+  if(aptcfg->FindB(PACKAGE "::Auto-Upgrade", false) && do_initselections)
     mark_all_upgradable(aptcfg->FindB("Auto-Install", true),
 			NULL);
 
   duplicate_cache(&backup_state);
 
+  Prog.Done();
+
   return true;
 }
 
@@ -401,13 +435,21 @@ void aptitudeDepCache::mark_all_upgradable(bool with_autoinst,
 	      else
 		{
 		  estate.selection_state=pkgCache::State::Install;
-		  if(state.Status>0)
-		    MarkInstall(i, do_autoinstall);
+		  if(state.Status>0 && !is_held(i))
+		    {
+		      dirty=true;
+
+		      MarkInstall(i, do_autoinstall);
+		    }
 		}
 	      break;
 	    case pkgCache::State::Install:
-	      if(state.Status>0)
-		MarkInstall(i, do_autoinstall);
+	      if(state.Status>0 && !is_held(i))
+		{
+		  dirty=true;
+
+		  MarkInstall(i, do_autoinstall);
+		}
 	    default:
 	      break;
 	    }
@@ -458,28 +500,35 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
       for(PkgIterator i=PkgBegin(); !i.end(); i++)
 	if(!i.VersionList().end())
 	  {
-	    aptitude_state &state=get_ext_state(i);
+	    StateCache &state=(*this)[i];
+	    aptitude_state &estate=get_ext_state(i);
 	    char buf[400];
 	    int len;
 
+	    string forbidstr=!estate.forbidver.empty()
+	      ? "ForbidVer: "+estate.forbidver+"\n":"";
+
+	    bool upgrade=(!i.CurrentVer().end()) && state.Install();
+	    string upgradestr=upgrade ? "Upgrade: yes\n" : "";
+
 	    // HACK: only write the version-string if we are writing
 	    //      to a non-default file.  ICKY.  MUST FIX.
-	    string tailstr=status_fname && !state.candver.empty()
-	      ?"Version: "+state.candver+"\n":"";
+	    string tailstr=status_fname && !estate.candver.empty()
+	      ?"Version: "+estate.candver+"\n":"";
 
-	    if(state.reinstall)
-	      tailstr+="ReInstall: yes\n\n";
-	    else
-	      tailstr+="\n";
+	    tailstr+="\n";
 
 	    len=snprintf(buf,
 			 400,
-			 "Package: %s\nUnseen: %s\nState: %i\nDselect-State: %i\nLast-Change: %i\n%s",
+			 "Package: %s\nUnseen: %s\nState: %i\nDselect-State: %i\nLast-Change: %i\nRemove-Reason: %i\n%s%s%s",
 			 i.Name(),
-			 state.new_package?"yes":"no",
-			 state.selection_state,
+			 estate.new_package?"yes":"no",
+			 estate.selection_state,
 			 i->SelectedState,
-			 state.installremove_reason,
+			 estate.install_reason,
+			 estate.remove_reason,
+			 upgradestr.c_str(),
+			 forbidstr.c_str(),
 			 tailstr.c_str());
 	    if(len>=399)
 	      {
@@ -504,6 +553,8 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 	    prog.OverallProgress(num, Head().PackageCount, 1, _("Writing extended state information"));
 	  }
 
+      prog.OverallProgress(Head().PackageCount, Head().PackageCount, 1, _("Writing extended state information"));
+
       if(newstate.Failed())
 	// This is /probably/ redundant, but paranoia never hurts.
 	{
@@ -512,6 +563,8 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 
 	  if(!status_fname)
 	    unlink((statefile+".new").c_str());
+
+	  prog.Done();
 	  return false;
 	}
       newstate.Close();
@@ -523,11 +576,13 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 	    {
 	      _error->Errno("save_selection_list", _("couldn't replace old state file"));
 	      unlink((statefile+".new").c_str());
+	      prog.Done();
 	      return false;
 	    }
 	}
     }
 
+  prog.Done();
   return true;
 }
 
@@ -557,7 +612,10 @@ void aptitudeDepCache::forget_new(undoable **undoer)
 
 undoable *aptitudeDepCache::state_restorer(PkgIterator pkg, StateCache &state, aptitude_state &ext_state)
 {
-  return new apt_undoer(pkg, state.Mode, state.iFlags, ext_state.installremove_reason, ext_state.selection_state, this);
+  return new apt_undoer(pkg, state.Mode, state.iFlags,
+			ext_state.install_reason, ext_state.remove_reason,
+			ext_state.selection_state,
+			ext_state.forbidver, this);
 }
 
 void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickies)
@@ -570,8 +628,16 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
       if(PkgState[pkg->ID].Mode!=backup_state.PkgState[pkg->ID].Mode ||
 	 package_states[pkg->ID].selection_state!=backup_state.AptitudeState[pkg->ID].selection_state ||
 	 package_states[pkg->ID].reinstall!=backup_state.AptitudeState[pkg->ID].reinstall ||
-	 package_states[pkg->ID].installremove_reason!=backup_state.AptitudeState[pkg->ID].installremove_reason)
+	 package_states[pkg->ID].install_reason!=backup_state.AptitudeState[pkg->ID].install_reason ||
+	 package_states[pkg->ID].remove_reason!=backup_state.AptitudeState[pkg->ID].remove_reason)
 	{
+	  int n=pkg->ID;
+	  n=n;
+	  char curM=PkgState[pkg->ID].Mode;
+	  curM=curM;
+	  char oldM=backup_state.PkgState[pkg->ID].Mode;
+	  oldM=oldM;
+
 	  if(alter_stickies &&
 	     PkgState[pkg->ID].Mode!=backup_state.PkgState[pkg->ID].Mode &&
 	     package_states[pkg->ID].selection_state==backup_state.AptitudeState[pkg->ID].selection_state)
@@ -584,23 +650,30 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
 		  if(package_states[pkg->ID].selection_state!=pkgCache::State::DeInstall)
 		    {
 		      if(!pkg.CurrentVer().end())
-			package_states[pkg->ID].installremove_reason=libapt;
+			package_states[pkg->ID].remove_reason=libapt;
 
 		      package_states[pkg->ID].selection_state=pkgCache::State::DeInstall;
 		    }
 		  break;
 		case ModeKeep:
-		  package_states[pkg->ID].selection_state=pkgCache::State::Hold;
+		  // If this was going to be deleted, assume it was
+		  // automatically marked to be installed.
+		  if(package_states[pkg->ID].selection_state==pkgCache::State::DeInstall ||
+		     package_states[pkg->ID].selection_state==pkgCache::State::Purge)
+		    package_states[pkg->ID].install_reason=libapt;
+
+		  package_states[pkg->ID].selection_state=pkgCache::State::Install;
 		  break;
 		case ModeInstall:
 		  if(package_states[pkg->ID].selection_state!=pkgCache::State::Install)
 		    {
 		      package_states[pkg->ID].selection_state=pkgCache::State::Install;
-		      // Only set it to not be manual if it's a new install.
-		      // (ie, automagically upgraded packages won't suddenly
-		      // be autoremovable)
-		      if(pkg.CurrentVer().end())
-			package_states[pkg->ID].installremove_reason=libapt;
+		      // Only set it to not be manual if it's a new
+		      // install, or the package was going to be
+		      // removed.
+		      if(pkg.CurrentVer().end() ||
+			 backup_state.PkgState[pkg->ID].Delete())
+			package_states[pkg->ID].install_reason=libapt;
 		    }
 		  break;
 		}
@@ -622,6 +695,12 @@ void aptitudeDepCache::internal_mark_install(const PkgIterator &Pkg,
 {
   dirty=true;
 
+  // Only change the install reason if the package was not previously
+  // installed OR it was going to be removed because it was gc'ed.
+  if((Pkg.CurrentVer().end() && !(*this)[Pkg].Install()) ||
+     (!Pkg.CurrentVer().end() && get_ext_state(Pkg).garbage))
+    get_ext_state(Pkg).install_reason=manual;
+
   if(!ReInstall)
     pkgDepCache::MarkInstall(Pkg, AutoInst);
   else
@@ -631,7 +710,7 @@ void aptitudeDepCache::internal_mark_install(const PkgIterator &Pkg,
 
   get_ext_state(Pkg).selection_state=pkgCache::State::Install;
   get_ext_state(Pkg).reinstall=ReInstall;
-  get_ext_state(Pkg).installremove_reason=manual;
+  get_ext_state(Pkg).forbidver="";
 
   if(group_level==0)
     {
@@ -652,16 +731,21 @@ void aptitudeDepCache::internal_mark_delete(const PkgIterator &Pkg,
 {
   dirty=true;
 
+  bool previously_to_delete=(*this)[Pkg].Delete();
+
   pkgDepCache::MarkDelete(Pkg, Purge);
   pkgDepCache::SetReInstall(Pkg, false);
 
   get_ext_state(Pkg).selection_state=(Purge?pkgCache::State::Purge:pkgCache::State::DeInstall);
   get_ext_state(Pkg).reinstall=false;
 
-  if(unused_delete)
-    get_ext_state(Pkg).installremove_reason=unused;
-  else
-    get_ext_state(Pkg).installremove_reason=manual;
+  if(!previously_to_delete)
+    {
+      if(unused_delete)
+	get_ext_state(Pkg).remove_reason=unused;
+      else
+	get_ext_state(Pkg).remove_reason=manual;
+    }
 
   // Argh argh argh.  Doing any of this when there's a mark-and-sweep
   // running causes extreme screwiness.
@@ -684,6 +768,11 @@ void aptitudeDepCache::internal_mark_keep(const PkgIterator &Pkg, bool Soft, boo
   pkgDepCache::SetReInstall(Pkg, false);
   get_ext_state(Pkg).reinstall=false;
 
+  // On a garbage package, this has the effect of making it manually
+  // installed.
+  if(!Pkg.CurrentVer().end() && get_ext_state(Pkg).garbage)
+    get_ext_state(Pkg).install_reason=manual;
+
   if(Pkg.CurrentVer().end())
     {
       if((*this)[Pkg].iFlags&Purge)
@@ -696,8 +785,6 @@ void aptitudeDepCache::internal_mark_keep(const PkgIterator &Pkg, bool Soft, boo
   else
     get_ext_state(Pkg).selection_state=pkgCache::State::Install;
 
-  get_ext_state(Pkg).installremove_reason=manual;
-
   if(group_level==0)
     {
       if(do_mark_and_sweep)
@@ -709,7 +796,6 @@ void aptitudeDepCache::internal_mark_keep(const PkgIterator &Pkg, bool Soft, boo
     }
 }
 
-#ifdef HAVE_LIBAPT_PKG3
 void aptitudeDepCache::set_candidate_version(const VerIterator &ver,
 					     undo_group *undo)
 {
@@ -734,7 +820,32 @@ void aptitudeDepCache::set_candidate_version(const VerIterator &ver,
       mark_and_sweep(undo);
     }
 }
-#endif
+
+void aptitudeDepCache::forbid_upgrade(const PkgIterator &pkg,
+				      string verstr, undo_group *undo)
+{
+  aptitude_state &estate=get_ext_state(pkg);
+
+  if(verstr!=estate.forbidver)
+    {
+      pkgCache::VerIterator candver=(*this)[pkg].CandidateVerIter(*this);
+
+      dirty=true;
+
+      estate.forbidver=verstr;
+      if(!candver.end() && candver.VerStr()==verstr && (*this)[pkg].Install())
+	MarkKeep(pkg, false);
+
+      if(group_level==0)
+	{
+	  mark_and_sweep(undo);
+
+	  cleanup_after_change(undo, false);
+
+	  duplicate_cache(&backup_state);
+	}
+    }
+}
 
 void aptitudeDepCache::mark_single_install(const PkgIterator &Pkg, undo_group *undo)
 {
@@ -761,7 +872,7 @@ void aptitudeDepCache::mark_auto_installed(const PkgIterator &Pkg,
 {
   dirty=true;
 
-  get_ext_state(Pkg).installremove_reason=set_auto?user_auto:manual;
+  get_ext_state(Pkg).install_reason=set_auto?user_auto:manual;
 
   if(group_level==0)
     {
@@ -785,7 +896,7 @@ bool aptitudeDepCache::all_upgrade(undo_group *undo)
       if((*this)[pkg].Install())
 	fixer.Protect(pkg);
 
-      if(get_ext_state(pkg).selection_state!=pkgCache::State::Hold &&
+      if(!is_held(pkg) &&
 	 !pkg.CurrentVer().end() && !(*this)[pkg].Install())
 	MarkInstall(pkg, false);
     }
@@ -828,11 +939,7 @@ bool aptitudeDepCache::try_fix_broken(pkgProblemResolver &fixer, undo_group *und
 
 bool aptitudeDepCache::try_fix_broken(undo_group *undo)
 {
-#ifndef HAVE_LIBAPT_PKG3
-  pkgProblemResolver fixer(*this);
-#else
   pkgProblemResolver fixer(this);
-#endif
   for(pkgCache::PkgIterator i=PkgBegin(); !i.end(); i++)
     {
       fixer.Clear(i);
@@ -900,7 +1007,11 @@ void aptitudeDepCache::duplicate_cache(apt_state_snapshot *target)
 
   memcpy(target->PkgState, PkgState, sizeof(StateCache)*Head().PackageCount);
   memcpy(target->DepState, DepState, sizeof(char)*Head().DependsCount);
-  memcpy(target->AptitudeState, package_states, sizeof(aptitude_state)*Head().PackageCount);
+  // memcpy doesn't work here because the aptitude_state structure
+  // contains a std::string.  (would it be worthwhile/possible to
+  // change things so that it doesn't?)
+  for(unsigned int i=0; i<Head().PackageCount; ++i)
+    target->AptitudeState[i]=package_states[i];
 
   target->iUsrSize=iUsrSize;
   target->iDownloadSize=iDownloadSize;
@@ -944,7 +1055,11 @@ void aptitudeDepCache::restore_apt_state(const apt_state_snapshot *snapshot)
 {
   memcpy(PkgState, snapshot->PkgState, sizeof(StateCache)*Head().PackageCount);
   memcpy(DepState, snapshot->DepState, sizeof(char)*Head().DependsCount);
-  memcpy(package_states, snapshot->AptitudeState, sizeof(aptitude_state)*Head().PackageCount);
+  // memcpy doesn't work here because the aptitude_state structure
+  // contains a std::string.  (would it be worthwhile/possible to
+  // change things so that it doesn't?)
+  for(unsigned int i=0; i<Head().PackageCount; ++i)
+    package_states[i]=snapshot->AptitudeState[i];
 
   iUsrSize=snapshot->iUsrSize;
   iDownloadSize=snapshot->iDownloadSize;
@@ -964,21 +1079,34 @@ void aptitudeDepCache::restore_apt_state(const apt_state_snapshot *snapshot)
 void aptitudeDepCache::mark_package(const PkgIterator &pkg,
 				    const VerIterator &ver)
 {
+  pkgDepCache::StateCache &state=(*this)[pkg];
+  aptitude_state &estate=get_ext_state(pkg);
+
+  // If a package was garbage-collected but is not being marked, we
+  // should re-mark it.  (note: undo==NULL is actually correct;
+  // mark_and_sweep *never* stores undo info....)
+  if(state.Delete() && estate.remove_reason==unused)
+    {
+      mark_install(pkg, false, false, NULL);
+      MarkKeep(pkg);
+    }
+
   if(!((ver==pkg.CurrentVer() && PkgState[pkg->ID].Keep()) ||
-       (ver==PkgState[pkg->ID].InstVerIter(GetCache()) &&
-	PkgState[pkg->ID].Install())))
+       (ver==state.InstVerIter(GetCache()) &&
+	state.Install())))
     return;
 
-  if(package_states[pkg->ID].marked)
+  if(estate.marked)
     return;
 
-  package_states[pkg->ID].marked=true;
+  estate.marked=true;
 
   if(!ver.end())
     {
       for(DepIterator d=ver.DependsList(); !d.end(); ++d)
 	{
 	  if(d->Type==pkgCache::Dep::Depends ||
+	     d->Type==pkgCache::Dep::PreDepends ||
 	     (aptcfg->FindB(PACKAGE "::Recommends-Important", true) &&
 	      d->Type==pkgCache::Dep::Recommends) ||
 	     (aptcfg->FindB(PACKAGE "::Suggests-Important", false) &&
@@ -986,20 +1114,12 @@ void aptitudeDepCache::mark_package(const PkgIterator &pkg,
 	    {
 	      // Try all versions of this package.
 	      for(VerIterator V=d.TargetPkg().VersionList(); !V.end(); ++V)
-#ifndef HAVE_LIBAPT_PKG3
-		if(pkgCheckDep(d.TargetVer(), V.VerStr(), d->CompareOp))
-#else
 		if(_system->VS->CheckDep(V.VerStr(), d->CompareOp, d.TargetVer()))
-#endif
 		  mark_package(V.ParentPkg(), V);
 
 	      // Now try virtual packages
 	      for(PrvIterator prv=d.TargetPkg().ProvidesList(); !prv.end(); ++prv)
-#ifndef HAVE_LIBAPT_PKG3
-		if(pkgCheckDep(d.TargetVer(), prv.ProvideVersion(), d->CompareOp))
-#else
 		if(_system->VS->CheckDep(prv.ProvideVersion(), d->CompareOp, d.TargetVer()))
-#endif
 		  mark_package(prv.OwnerPkg(), prv.OwnerVer());
 	    }
 	}
@@ -1027,7 +1147,7 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
 
   for(pkgCache::PkgIterator p=PkgBegin(); !p.end(); ++p)
     {
-      if(package_states[p->ID].installremove_reason==manual ||
+      if(package_states[p->ID].install_reason==manual ||
 	 (p->Flags & pkgCache::Flag::Essential) ||
 	 (matcher && ((PkgState[p->ID].Keep() &&
 		       !p.CurrentVer().end() &&
@@ -1042,116 +1162,43 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
 	}
     }
 
-  // Be sure not to re-delete already deleted packages.
   for(pkgCache::PkgIterator p=PkgBegin(); !p.end(); ++p)
-    if(!package_states[p->ID].marked)
-      {
-	package_states[p->ID].garbage=true;
+    {
+      StateCache &state=(*this)[p];
+      aptitude_state &estate=get_ext_state(p);
 
-	if((!p.CurrentVer().end() || PkgState[p->ID].Install()) &&
-	   !PkgState[p->ID].Delete() &&
-	   aptcfg->FindB(PACKAGE "::Delete-Unused", true))
-	  mark_delete(p, false, true, undo);
-      }
+      if(!estate.marked)
+	{
+	  estate.garbage=true;
+
+	  // Be sure not to re-delete already deleted packages.
+	  if((!p.CurrentVer().end() || state.Install()) &&
+	     !state.Delete() &&
+	     aptcfg->FindB(PACKAGE "::Delete-Unused", true))
+	    mark_delete(p, false, true, undo);
+	}
+    }
 
   mark_and_sweep_in_progress=false;
 
   delete matcher;
 }
 
-#ifndef HAVE_LIBAPT_PKG3
-aptitudeCacheFile::aptitudeCacheFile():Map(NULL), Cache(NULL), Lock(NULL)
-{
-}
-#else
 aptitudeCacheFile::aptitudeCacheFile()
   :Map(NULL), Cache(NULL), DCache(NULL), have_system_lock(false), Policy(NULL)
 {
 }
-#endif
 
 aptitudeCacheFile::~aptitudeCacheFile()
 {
   delete Cache;
   delete Map;
-#ifndef HAVE_LIBAPT_PKG3
-  delete Lock;
-#else
   ReleaseLock();
 
   delete DCache;
   delete Policy;
-#endif
 }
 
-#ifndef HAVE_LIBAPT_PKG3
-bool aptitudeCacheFile::Open(OpProgress &Progress, bool do_initselections,
-			     bool WithLock, const char *status_fname)
-{
-  if(WithLock)
-    Lock=new pkgDpkgLock;
-
-  if(_error->PendingError())
-    return false;
-
-  pkgSourceList List;
-  if(!List.ReadMainList())
-    return _error->Error(_("The list of sources could not be read."));
-
-  if(WithLock)
-    {
-      pkgMakeStatusCache(List, Progress);
-      if(_error->PendingError())
-	return _error->Error(_("The package lists or status file could not be parsed or opened."));
-      if(!_error->empty())
-	_error->Warning(_("You may want to update the package lists to correct these missing files"));
-
-      FileFd File(_config->FindFile("Dir::Cache::pkgcache"), FileFd::ReadOnly);
-      if(_error->PendingError())
-	return false;
-
-      Map=new MMap(File, MMap::Public|MMap::ReadOnly);
-      if(_error->PendingError())
-	return false;
-    }
-  else
-    {
-      Map=pkgMakeStatusCacheMem(List, Progress);
-      Progress.Done();
-      if(!Map)
-	return false;
-    }
-
-  Cache=new aptitudeDepCache(*Map, Progress, WithLock,
-			     do_initselections, status_fname);
-  Progress.Done();
-  if(_error->PendingError())
-    return false;
-
-  return true;
-}
-
-bool aptitudeCacheFile::GainLock()
-{
-  if(Lock)
-    return true;
-
-  // Hacky but this is an obsolete library that I'm using..just do something
-  // that works
-  _error->Discard();
-
-  Lock=new pkgDpkgLock;
-
-  if(_error->PendingError())
-    {
-      delete Lock;
-      return false;
-    }
-  else
-    return true;
-}
-
-#else
 bool aptitudeCacheFile::Open(OpProgress &Progress, bool do_initselections,
 			     bool WithLock, const char *status_fname)
 {
@@ -1177,7 +1224,6 @@ bool aptitudeCacheFile::Open(OpProgress &Progress, bool do_initselections,
   if(!Res)
     return _error->Error(_("The package lists or status file could not be parsed or opened."));
 
-  // Jason says this is bad.  I agree. :)
   if(!_error->empty())
     _error->Warning(_("You may want to update the package lists to correct these missing files"));
 
@@ -1225,4 +1271,13 @@ bool aptitudeCacheFile::GainLock()
   return true;
 }
 
-#endif
+bool aptitudeDepCache::is_held(const PkgIterator &pkg)
+{
+  aptitude_state state=get_ext_state(pkg);
+
+  pkgCache::VerIterator candver=(*apt_cache_file)[pkg].CandidateVerIter(*apt_cache_file);
+
+  return !pkg.CurrentVer().end() &&
+    (state.selection_state == pkgCache::State::Hold ||
+     (!candver.end() && candver.VerStr() == state.forbidver));
+}
