@@ -22,21 +22,10 @@
 #include <apt-pkg/pkgcache.h>
 #include <apt-pkg/error.h>
 
-#ifndef HAVE_LIBAPT_PKG3
-#include "aptcache.h"
-#endif
-
-#ifdef HAVE_LIBAPT_PKG3
 bool aptitudePolicy::IsImportantDep(pkgCache::DepIterator dep)
 {
   if(pkgPolicy::IsImportantDep(dep))
     return true;
-#else
-  bool aptitudeDepCache::IsImportantDep(pkgCache::DepIterator dep)
-{
-  if(pkgDepCache::IsImportantDep(dep))
-    return true;
-#endif
 
   if(dep->Type==pkgCache::Dep::Recommends)
     return aptcfg->FindB(PACKAGE "::Recommends-Important", true);
