@@ -40,101 +40,122 @@
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/version.h>
 
+#include <assert.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <regex.h>
+#include <sys/types.h>
 
 using namespace std;
 
-inline const char *stristr(const char *haystack, const char *needle)
-  // There are no basic_string methods to efficiently compare two strings
-  // case-insensitively, and no string functions to do the same for
-  // const char * objects, so I'm writing my own.
-{
-  int lenh=strlen(haystack), lenn=strlen(needle);
-  if(lenn==0 || lenh==0)
-    return NULL;
-  for(int i=0; i<lenh-lenn+1; i++)
-    {
-      bool found=true;
-      for(int j=0; j<lenn; j++)
-	if(tolower(haystack[i+j])!=tolower(needle[j]))
-	  {
-	    found=false;
-	    break;
-	  }
-      if(found)
-	return haystack+i;
-    }
-  return NULL;
-}
-
-inline bool string_matches(string pattern, string s, bool match_all)
+inline bool string_matches(regex_t *pattern, string s)
   // Altering this to support globs or regexps is all that's needed to add
   // support for those to the program (if that's done, the inline should be
   // removed :) )
-  //
-  //  If match_all is true, the string must be an exact match for the pattern;
-  // otherwise, any match is allowed.
 {
-  if(match_all)
-    return pattern==s;
-  else
-    return stristr(s.c_str(), pattern.c_str())!=NULL;
+  regmatch_t amount;
+
+  return !regexec(pattern, s.c_str(), 1, &amount, 0);
 }
 
-typedef bool (*mfunc)(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver, bool match_all);
+typedef bool (*mfunc)(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver);
 
 template<mfunc f>
 class pkg_string_matcher:public pkg_matcher
 {
-  string s;
-  bool match_all;
+  regex_t pattern;
+
+  pkg_string_matcher()
+  {
+  }
+
+  bool compile(string _pattern, bool flag_errors)
+  {
+    int err=regcomp(&pattern, _pattern.c_str(), REG_ICASE|REG_EXTENDED|REG_NOSUB);
+    if(err!=0)
+      {
+	if(flag_errors)
+	  {
+	    size_t needed=regerror(err, &pattern, NULL, 0);
+
+	    char *buf=new char[needed+1];
+
+	    regerror(err, &pattern, buf, needed+1);
+
+	    _error->Error("Regex compilation error: %s", buf);
+
+	    delete buf;
+	  }
+
+	return false;
+      }
+    else
+      return true;
+  }
 public:
-  pkg_string_matcher(string _s, bool _match_all):s(_s), match_all(_match_all) {}
+  static pkg_string_matcher<f> *init(string _pattern,
+				     bool flag_errors)
+  {
+    pkg_string_matcher<f> *rval=new pkg_string_matcher<f>;
+
+    if(!rval->compile(_pattern, flag_errors))
+      {
+	delete rval;
+	return NULL;
+      }
+    else
+      return rval;
+  }
+
+  ~pkg_string_matcher()
+  {
+    regfree(&pattern);
+  }
+
   bool matches(pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
   {
-    return f(s, pkg, ver, match_all);
+    return f(&pattern, pkg, ver);
   }
 };
 
-bool name_matches(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver, bool match_all)
+bool name_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
 {
-  return string_matches(s, pkg.Name(), match_all);
+  return string_matches(pattern, pkg.Name());
 }
 typedef pkg_string_matcher<name_matches> pkg_name_matcher;
 
-bool description_matches(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver, bool match_all)
+bool description_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
 {
-  return !ver.end() && string_matches(s, apt_package_records->Lookup(ver.FileList()).LongDesc(), match_all);
+  return !ver.end() && string_matches(pattern, apt_package_records->Lookup(ver.FileList()).LongDesc());
 }
 typedef pkg_string_matcher<description_matches> pkg_description_matcher;
 
-bool maintainer_matches(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver, bool match_all)
+bool maintainer_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
 {
-  return !ver.end() && string_matches(s, apt_package_records->Lookup(ver.FileList()).Maintainer(), match_all);
+  return !ver.end() && string_matches(pattern, apt_package_records->Lookup(ver.FileList()).Maintainer());
 }
 typedef pkg_string_matcher<maintainer_matches> pkg_maintainer_matcher;
 
-bool priority_matches(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver, bool match_all)
+bool priority_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
 {
   return !ver.end() && ver.PriorityType() &&
-    string_matches(s, ver.PriorityType(), match_all);
+    string_matches(pattern, ver.PriorityType());
 }
 typedef pkg_string_matcher<priority_matches> pkg_priority_matcher;
 
-bool section_matches(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver, bool match_all)
+bool section_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
 {
-  return !ver.end() && ver.Section() && string_matches(s, ver.Section(), match_all);
+  return !ver.end() && ver.Section() && string_matches(pattern, ver.Section());
 }
 typedef pkg_string_matcher<section_matches> pkg_section_matcher;
 
-bool version_matches(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver, bool match_all)
+bool version_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
 {
-  return !ver.end() && ver.VerStr() && string_matches(s, ver.VerStr(), match_all);
+  return !ver.end() && ver.VerStr() && string_matches(pattern, ver.VerStr());
 }
 typedef pkg_string_matcher<version_matches> pkg_version_matcher;
 
-bool task_matches(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver, bool match_all)
+bool task_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
 {
   list<string> *l=get_tasks(pkg);
 
@@ -144,7 +165,7 @@ bool task_matches(string s, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver
   for(list<string>::iterator i=l->begin();
       i!=l->end();
       ++i)
-    if(string_matches(s, *i, match_all))
+    if(string_matches(pattern, *i))
       return true;
 
   return false;
@@ -155,18 +176,17 @@ typedef pkg_string_matcher<task_matches> pkg_task_matcher;
 // available files (for all versions) match the given criteria.
 //
 //  Should I use templates?
-bool origin_matches(string s,
+bool origin_matches(regex_t *pattern,
 		    pkgCache::PkgIterator pkg,
-		    pkgCache::VerIterator ver, bool match_all)
+		    pkgCache::VerIterator ver)
 {
   for(pkgCache::VerIterator v=pkg.VersionList(); !v.end(); ++v)
     for(pkgCache::VerFileIterator f=v.FileList(); !f.end(); ++f)
       {
 	pkgCache::PkgFileIterator cur=f.File();
 
-	if(!cur.end() && cur.Origin() && string_matches(s,
-							cur.Origin(),
-							match_all))
+	if(!cur.end() && cur.Origin() && string_matches(pattern,
+							cur.Origin()))
 	  return true;
       }
 
@@ -174,9 +194,9 @@ bool origin_matches(string s,
 }
 typedef pkg_string_matcher<origin_matches> pkg_origin_matcher;
 
-bool archive_matches(string s,
+bool archive_matches(regex_t *pattern,
 		     pkgCache::PkgIterator pkg,
-		     pkgCache::VerIterator ver, bool match_all)
+		     pkgCache::VerIterator ver)
 {
   if(ver.end() || ver.FileList().end())
     return false;
@@ -186,9 +206,8 @@ bool archive_matches(string s,
       {
 	pkgCache::PkgFileIterator cur=f.File();
 
-	if(!cur.end() && cur.Archive() && string_matches(s,
-							 cur.Archive(),
-							 match_all))
+	if(!cur.end() && cur.Archive() && string_matches(pattern,
+							 cur.Archive()))
 	  return true;
       }
 
@@ -203,7 +222,7 @@ public:
   {
     return
       (!pkg.CurrentVer().end() || (*apt_cache_file)[pkg].Install()) &&
-      ((*apt_cache_file)->get_ext_state(pkg).installremove_reason!=aptitudeDepCache::manual);
+      ((*apt_cache_file)->get_ext_state(pkg).install_reason!=aptitudeDepCache::manual);
   }
 };
 
@@ -539,11 +558,53 @@ pkgCache::Dep::DepType parse_deptype(string s)
 }
 
 pkg_matcher *parse_condition_list(string s, unsigned int &loc,
-				  bool &exact_match, bool flag_errors);
+				  bool flag_errors);
+
+// Returns a substring up to the first metacharacter, including escaped
+// metacharacters (parentheses, ~, |, and !)
+//
+// Advances loc to the first character of 's' following the escaped string.
+std::string parse_substr(string s, unsigned int &loc)
+{
+  std::string rval="";
+  bool done=false;
+
+  do
+    {
+      while(loc<s.size() &&
+	    s[loc]!='(' &&
+	    s[loc]!=')' &&
+	    s[loc]!='!' &&
+	    s[loc]!='~' &&
+	    s[loc]!='|')
+	{
+	  rval+=s[loc];
+	  ++loc;
+	}
+
+      // We quit because we ran off the end of the string or saw a
+      // metacharacter.  If the latter case and it was a tilde-escape,
+      // add the escaped character to the string and continue.
+      if(loc<s.size()-1 && s[loc]=='~' &&
+	 (s[loc+1]=='(' || s[loc+1]==')' ||
+	  s[loc+1]=='!' || s[loc+1]=='~' ||
+	  s[loc+1]=='|'))
+	{
+	  rval+=s[loc+1];
+	  loc+=2;
+	}
+      else
+	done=true;
+    } while(!done);
+
+  return rval;
+}
 
 pkg_matcher *parse_atom(string s, unsigned int &loc,
-			bool &exact_match, bool flag_errors)
+			bool flag_errors)
 {
+  std::string substr;
+
   while(loc<s.size() && isspace(s[loc]))
     loc++;
 
@@ -552,7 +613,7 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
       if(s[loc]=='!')
 	{
 	  loc++;
-	  pkg_matcher *atom=parse_atom(s, loc, exact_match, flag_errors);
+	  pkg_matcher *atom=parse_atom(s, loc, flag_errors);
 	  if(!atom)
 	    return NULL;
 	  return new pkg_not_matcher(atom);
@@ -562,7 +623,7 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 	{
 	  loc++;
 	  pkg_matcher *lst=parse_condition_list(s, loc,
-						exact_match, flag_errors);
+						flag_errors);
 	  if(!lst)
 	    return NULL;
 
@@ -584,7 +645,7 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 	  if(loc+1==s.size())
 	    {
 	      loc++;
-	      return new pkg_name_matcher("~", false);
+	      return pkg_name_matcher::init("~", flag_errors);
 	    }
 	  else
 	    {
@@ -614,9 +675,6 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 		  return new pkg_installed_matcher;
 		case 'E':
 		  return new pkg_essential_matcher;
-		case 'e':
-		  exact_match=!exact_match;
-		  break;
 		case 'M':
 		  return new pkg_auto_matcher;
 		case 'N':
@@ -626,7 +684,7 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 		case 'D':
 		case 'C':
 		  {
-		    pkg_matcher *m=parse_atom(s, loc, exact_match, flag_errors);
+		    pkg_matcher *m=parse_atom(s, loc, flag_errors);
 
 		    if(!m)
 		      return NULL;
@@ -637,63 +695,51 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 		      return new pkg_dep_matcher(m, pkg_dep_matcher::conflict);
 		  }
 		default:
-		  while(loc<s.size() &&
-			s[loc]!='(' &&
-			s[loc]!=')' &&
-			s[loc]!='!' &&
-			s[loc]!=')' &&
-			s[loc]!='~' &&
-			s[loc]!='|')
-		    loc++;
+		  substr=parse_substr(s, loc);
 		  switch(s[prevloc+1])
 		    {
-		    case '~':
-		      return new pkg_name_matcher(s.substr(prevloc+1, loc-prevloc-1), exact_match);
 		    case 'a':
 		      {
-			string type=s.substr(prevloc+2, loc-prevloc-2);
-
 			// Match packages to be installed
-			if(!strcasecmp(type.c_str(), "install"))
+			if(!strcasecmp(substr.c_str(), "install"))
 			  return new pkg_action_matcher(pkg_install, false);
 
 			// Match packages to be upgraded
-			else if(!strcasecmp(type.c_str(), "upgrade"))
+			else if(!strcasecmp(substr.c_str(), "upgrade"))
 			  return new pkg_action_matcher(pkg_upgrade, false);
 
 			// Match packages to be removed OR purged
-			else if(!strcasecmp(type.c_str(), "remove"))
+			else if(!strcasecmp(substr.c_str(), "remove"))
 			  return new pkg_action_matcher(pkg_remove, false);
 
 			// Match packages to be purged
-			else if(!strcasecmp(type.c_str(), "purge"))
+			else if(!strcasecmp(substr.c_str(), "purge"))
 			  return new pkg_action_matcher(pkg_remove, true);
 
 			// Match packages to be reinstalled
-			else if(!strcasecmp(type.c_str(), "reinstall"))
+			else if(!strcasecmp(substr.c_str(), "reinstall"))
 			  return new pkg_action_matcher(pkg_reinstall, false);
 
 			// Match held packages
-			else if(!strcasecmp(type.c_str(), "hold"))
+			else if(!strcasecmp(substr.c_str(), "hold"))
 			  return new pkg_action_matcher(pkg_hold, false);
 
 			else
 			  {
 			    char buf[512];
 			    snprintf(buf, 512, _("Unknown action type: %s"),
-				     type.c_str());
+				     substr.c_str());
 			    if(flag_errors)
 			      _error->Error("%s", buf);
 			    return NULL;
 			  }
 		      }
 		    case 'A':
-		      return new pkg_archive_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_archive_matcher::init(substr, flag_errors);
 		    case 'B':
 		    case 'R':
 		      {
-			string type=s.substr(prevloc+2, loc-prevloc-2);
-			pkgCache::Dep::DepType ptype=parse_deptype(type);
+			pkgCache::Dep::DepType ptype=parse_deptype(substr);
 
 			if(ptype!=-1)
 			  {
@@ -709,28 +755,28 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 			  {
 			    char buf[512];
 			    snprintf(buf, 512, _("Unknown dependency type: %s"),
-				     type.c_str());
+				     substr.c_str());
 			    if(flag_errors)
 			      _error->Error("%s", buf);
 			    return NULL;
 			  }
 		      }
 		    case 'n':
-		      return new pkg_name_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_name_matcher::init(substr, flag_errors);
 		    case 'd':
-		      return new pkg_description_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_description_matcher::init(substr, flag_errors);
 		    case 'm':
-		      return new pkg_maintainer_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_maintainer_matcher::init(substr, flag_errors);
 		    case 'O':
-		      return new pkg_origin_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_origin_matcher::init(substr, flag_errors);
 		    case 'p':
-		      return new pkg_priority_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_priority_matcher::init(substr, flag_errors);
 		    case 's':
-		      return new pkg_section_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_section_matcher::init(substr, flag_errors);
 		    case 'V':
-		      return new pkg_version_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_version_matcher::init(substr, flag_errors);
 		    case 't':
-		      return new pkg_task_matcher(s.substr(prevloc+2, loc-prevloc-2), exact_match);
+		      return pkg_task_matcher::init(substr, flag_errors);
 		    default:
 		      {
 			char buf[512];
@@ -745,10 +791,8 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 	}
       else
 	{
-	  unsigned int prevloc=loc;
-	  while(loc<s.size() && s[loc]!='(' && s[loc]!=')' && s[loc]!='!' && s[loc]!=')' && s[loc]!='~' && s[loc]!='|')
-	    loc++;
-	  return new pkg_name_matcher(s.substr(prevloc, loc-prevloc), exact_match);
+	  return pkg_name_matcher::init(parse_substr(s, loc),
+					flag_errors);
 	}
     }
 
@@ -757,7 +801,7 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 }
 
 pkg_matcher *parse_and_group(string s, unsigned int &loc,
-			     bool &exact_match, bool flag_errors)
+			     bool flag_errors)
 {
   pkg_matcher *rval=NULL;
   while(loc<s.size() && isspace(s[loc]))
@@ -765,7 +809,7 @@ pkg_matcher *parse_and_group(string s, unsigned int &loc,
 
   while(loc<s.size() && s[loc]!='|' && s[loc]!=')')
     {
-      pkg_matcher *atom=parse_atom(s, loc, exact_match, flag_errors);
+      pkg_matcher *atom=parse_atom(s, loc, flag_errors);
       if(!atom)
 	{
 	  delete rval;
@@ -786,11 +830,11 @@ pkg_matcher *parse_and_group(string s, unsigned int &loc,
 }
 
 pkg_matcher *parse_condition_list(string s, unsigned int &loc,
-				  bool &exact_match, bool flag_errors)
+				  bool flag_errors)
   // This (and the grammer) should be optimized to avoid deep recursion
   // what does the above line mean? -- DNB 11/21/01
 {
-  pkg_matcher *grp=parse_and_group(s, loc, exact_match, flag_errors);
+  pkg_matcher *grp=parse_and_group(s, loc, flag_errors);
   if(!grp)
     return NULL;
 
@@ -803,7 +847,7 @@ pkg_matcher *parse_condition_list(string s, unsigned int &loc,
 	{
 	  loc++;
 	  pkg_matcher *grp2=parse_condition_list(s, loc,
-						 exact_match, flag_errors);
+						 flag_errors);
 	  if(!grp2)
 	    {
 	      delete grp;
@@ -830,7 +874,6 @@ pkg_matcher *parse_condition_list(string s, unsigned int &loc,
 pkg_matcher *parse_pattern(string s, bool flag_errors)
 {
   unsigned int loc=0;
-  bool exact_match=false;
 
   // Just filter blank strings out immediately.
   while(loc!=s.size() && isspace(s[loc]))
@@ -839,7 +882,7 @@ pkg_matcher *parse_pattern(string s, bool flag_errors)
   if(loc==s.size())
     return NULL;
 
-  pkg_matcher *rval=parse_condition_list(s, loc, exact_match, flag_errors);
+  pkg_matcher *rval=parse_condition_list(s, loc, flag_errors);
 
   if(rval==NULL)
     return NULL;
