@@ -8,6 +8,7 @@
 #include <ui.h>
 
 #include <sigc++/bind.h>
+#include <sigc++/object_slot.h>
 
 #include <vscreen/config/keybindings.h>
 #include <vscreen/config/colors.h>
@@ -62,7 +63,7 @@ size cmine::size_request()
 
 void cmine::update_header()
 {
-  timeout_num=vscreen_addtimeout(slot(this, &cmine::update_header), 500);
+  timeout_num=vscreen_addtimeout(slot(*this, &cmine::update_header), 500);
 
   vscreen_update();
 };
@@ -81,18 +82,18 @@ void cmine::paint_header()
       if(board->get_state()==mine_board::playing)
 	snprintf(buf,
 		 sizeof(buf),
-		 "%i/%i mines  %d %s",
+		 _("%i/%i mines  %d %s"),
 		 board->get_nummines()-board->get_numflags(),
 		 board->get_nummines(),
 		 (int) board->get_duration(),
-		 board->get_duration()==1?"second":"seconds");
+		 board->get_duration()==1?_("second"):_("seconds"));
       else
 	snprintf(buf,
 		 sizeof(buf),
-		 "Minesweeper    %s in %d %s",
-		 board->get_state()==mine_board::won?"Won":"Lost",
+		 _("Minesweeper    %s in %d %s"),
+		 board->get_state()==mine_board::won?_("Won"):_("Lost"),
 		 (int) board->get_duration(),
-		 board->get_duration()==1?"second":"seconds");
+		 board->get_duration()==1?_("second"):_("seconds"));
 
       while(header.size()+strlen(buf)<(unsigned) width)
 	header+=' ';
@@ -107,7 +108,7 @@ void cmine::paint_header()
       display_header(header.c_str(), get_color("ScreenHeaderColor"));
     }
   else
-    display_header("Minesweeper", get_color("ScreenHeaderColor"));
+    display_header(_("Minesweeper"), get_color("ScreenHeaderColor"));
 }
 
 void cmine::do_load_game(string s)
@@ -156,7 +157,7 @@ void cmine::do_save_game(string s)
 	{
 	  char buf[512];
 
-	  snprintf(buf, 512, "Could not open file \"%s\"", s.c_str());
+	  snprintf(buf, 512, _("Could not open file \"%s\""), s.c_str());
 
 	  popup_widget(vs_dialog_ok(buf, NULL, get_color("Error")));
 	}
@@ -241,19 +242,19 @@ void cmine::do_custom_game()
 				    attr);
   vs_editline *minesedit=new vs_editline("");
 
-  vs_button *okbutton=new vs_button("Ok");
+  vs_button *okbutton=new vs_button(_("Ok"));
   okbutton->set_bg(attr);
-  vs_button *cancelbutton=new vs_button("Cancel");
+  vs_button *cancelbutton=new vs_button(_("Cancel"));
   cancelbutton->set_bg(attr);
 
   table->connect_key("Confirm", &global_bindings, okbutton->pressed.slot());
 
-  okbutton->pressed.connect(bind(bind(slot(this, &cmine::do_start_custom_game),
+  okbutton->pressed.connect(bind(bind(slot(*this, &cmine::do_start_custom_game),
 				      widthedit,
 				      minesedit),
 				 center,
 				 heightedit));
-  cancelbutton->pressed.connect(slot(center, &vscreen_widget::destroy));
+  cancelbutton->pressed.connect(slot(*center, &vscreen_widget::destroy));
 
   table->connect_key("Cancel", &global_bindings, cancelbutton->pressed.slot());
 
@@ -314,9 +315,9 @@ void cmine::do_new_game()
   hardbutton->set_bg(attr);
   custombutton->set_bg(attr);
 
-  vs_button *okbutton=new vs_button("Ok");
+  vs_button *okbutton=new vs_button(_("Ok"));
   okbutton->set_bg(attr);
-  vs_button *cancelbutton=new vs_button("Cancel");
+  vs_button *cancelbutton=new vs_button(_("Cancel"));
   cancelbutton->set_bg(attr);
 
   table->connect_key("Confirm", &global_bindings, okbutton->pressed.slot());
@@ -327,11 +328,11 @@ void cmine::do_new_game()
   grp->add_button(hardbutton, 2);
   grp->add_button(custombutton, 3);
 
-  okbutton->pressed.connect(bind(bind(slot(this, &cmine::do_continue_new_game),
+  okbutton->pressed.connect(bind(bind(slot(*this, &cmine::do_continue_new_game),
 				      grp),
 				 true,
 				 center));
-  cancelbutton->pressed.connect(bind(bind(slot(this, &cmine::do_continue_new_game),
+  cancelbutton->pressed.connect(bind(bind(slot(*this, &cmine::do_continue_new_game),
 					  grp),
 				     false,
 				     center));
@@ -387,7 +388,7 @@ void cmine::do_continue_new_game(bool start,
 cmine::cmine():board(NULL)
 {
   set_board(easy_game());
-  vscreen_addtimeout(slot(this, &cmine::update_header), 500);
+  vscreen_addtimeout(slot(*this, &cmine::update_header), 500);
   //set_status(_("n - New Game  Cursor keys - move cursor  f - flag  enter - check  ? - help"));
 }
 
@@ -456,28 +457,28 @@ void cmine::checkend()
 	      switch(rand()/(RAND_MAX/8))
 		{
 		case 0:
-		  type="sleep";
+		  type=_("sleep");
 		  break;
 		case 1:
-		  type="striking";
+		  type=_("striking");
 		  break;
 		case 2:
-		  type="death";
+		  type=_("death");
 		  break;
 		case 3:
-		  type="polymorph";
+		  type=_("polymorph");
 		  break;
 		case 4:
-		  type="magic missile";
+		  type=_("magic missile");
 		  break;
 		case 5:
-		  type="secret door detection";
+		  type=_("secret door detection");
 		  break;
 		case 6:
-		  type="invisibility";
+		  type=_("invisibility");
 		  break;
 		case 7:
-		  type="cold";
+		  type=_("cold");
 		  break;
 		}
 
@@ -595,14 +596,14 @@ bool cmine::handle_char(chtype ch)
   else if(bindings->key_matches(ch, "MineLoadGame"))
     prompt_string(_("Enter the filename to load: "),
 		  "",
-		  slot(this, &cmine::do_load_game),
+		  slot(*this, &cmine::do_load_game),
 		  NULL,
 		  NULL,
 		  &load_history);
   else if(bindings->key_matches(ch, "MineSaveGame"))
     prompt_string(_("Enter the filename to save: "),
 		  "",
-		  slot(this, &cmine::do_save_game),
+		  slot(*this, &cmine::do_save_game),
 		  NULL,
 		  NULL,
 		  &save_history);
