@@ -30,9 +30,7 @@
 #include <apt-pkg/strutl.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/version.h>
-#ifdef HAVE_LIBAPT_PKG3
 #include <apt-pkg/pkgsystem.h>
-#endif
 
 #include <unistd.h>
 
@@ -59,6 +57,7 @@ column_type_defaults pkg_item::pkg_columnizer::defaults[pkg_columnizer::numtypes
   {2, false, false},    // revdepcount
   {1, false, false},    // autoset
   {1, false, false},    // tagged
+  {10, true, true},     // archive
   {strlen(PACKAGE), false, false},  // progname
   {strlen(VERSION), false, false},  // progver
   {12, false, true},    // brokencount
@@ -74,7 +73,7 @@ column_type_defaults pkg_item::pkg_columnizer::defaults[pkg_columnizer::numtypes
 //
 // You can't set default widths for the program name and version here (those
 // strings aren't affected by translation, for one thing)
-const char *default_widths=N_("30 6 6 1 1 40 10 10 11 10 35 9 10 2 1 1 12 30 15 15");
+const char *default_widths=N_("30 6 6 1 1 40 10 10 11 10 35 9 10 2 1 1 10 12 30 15 15");
 
 const char *pkg_item::pkg_columnizer::column_names[pkg_columnizer::numtypes]=
   {N_("Package"),
@@ -111,6 +110,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	return column_disposition(pkg.Name(), basex);
       else
 	return column_disposition("", 0);
+
+      break;
     case installed_size:
       if(!visible_ver.end())
 	{
@@ -121,6 +122,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	}
       else
 	return column_disposition("", 0);	
+
+      break;
     case debsize:
       if(!visible_ver.end())
 	{
@@ -139,6 +142,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	return column_disposition(pkg.CurrentVer().VerStr(), 0);
       else
 	return column_disposition(_("<none>"), 0);
+
+      break;
     case candver:
       if(!pkg.end())
 	{
@@ -151,6 +156,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	}
       else
 	return column_disposition("", 0);
+
+      break;
     case stateflag:
       if(pkg.end())
 	return column_disposition("", 0);
@@ -179,6 +186,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	  default:
 	    return column_disposition("E", 0);
 	  }
+
+      break;
     case longstate:
       if(pkg.end())
 	return column_disposition("", 0);
@@ -207,15 +216,24 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	  default:
 	    return column_disposition(_("ERROR"), 0);
 	  }
+
+      break;
     case actionflag:
       {
 	if(pkg.end())
 	  return column_disposition("", 0);
 
-	aptitudeDepCache::StateCache state=(*apt_cache_file)[pkg];
-	if(state.Status!=2 && (*apt_cache_file)->get_ext_state(pkg).selection_state==pkgCache::State::Hold && !state.InstBroken())
-	  // FIXME -- is overriding InstBroken with this mess a good idea?
+	aptitudeDepCache::StateCache &state=(*apt_cache_file)[pkg];
+	aptitudeDepCache::aptitude_state &estate=(*apt_cache_file)->get_ext_state(pkg);
+	pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
+
+	if(state.Status!=2 &&
+	   (*apt_cache_file)->get_ext_state(pkg).selection_state==pkgCache::State::Hold &&
+	   !state.InstBroken())
 	  return column_disposition("h", 0);
+	else if(state.Upgradable() && !pkg.CurrentVer().end() &&
+		!candver.end() && candver.VerStr() == estate.forbidver)
+	  return column_disposition("F", 0);
 	else if(state.Delete())
 	  return column_disposition((state.iFlags&pkgDepCache::Purge)?"p":"d", 0);
 	else if(state.InstBroken())
@@ -231,14 +249,22 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	else
 	  return column_disposition(" ", 0);
       }
+
+      break;
     case longaction:
       {
 	if(pkg.end())
 	  return column_disposition("", 0);
 
 	aptitudeDepCache::StateCache state=(*apt_cache_file)[pkg];
-	if(state.Status!=2 && (state.Held() || (*apt_cache_file)->get_ext_state(pkg).selection_state==pkgCache::State::Hold) && !state.InstBroken())
+	aptitudeDepCache::aptitude_state &estate=(*apt_cache_file)->get_ext_state(pkg);
+	pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
+
+	if(state.Status!=2 && (state.Held() || estate.selection_state==pkgCache::State::Hold) && !state.InstBroken())
 	  return column_disposition(_("hold"), 0);
+	else if(state.Upgradable() && !pkg.CurrentVer().end() &&
+		!candver.end() && candver.VerStr() == estate.forbidver)
+	  return column_disposition("forbidden upgrade", 0);
 	else if(state.Delete())
 	  return column_disposition((state.iFlags&pkgDepCache::Purge)?_("purge"):_("delete"), 0);
 	else if(state.InstBroken())
@@ -254,6 +280,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	else
 	  return column_disposition(_("none"), 0);
       }
+
+      break;
     case description:
       if(!visible_ver.end() &&
 	 !visible_ver.FileList().end() &&
@@ -261,6 +289,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	return column_disposition(apt_package_records->Lookup(visible_ver.FileList()).ShortDesc(), 0);
       else
 	return column_disposition("", 0);
+
+      break;
     case maintainer:
       if(!visible_ver.end() &&
 	 !visible_ver.FileList().end() &&
@@ -268,6 +298,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	return column_disposition(apt_package_records->Lookup(visible_ver.FileList()).Maintainer(), 0);
       else
 	return column_disposition("", 0);
+
+      break;
     case priority:
       if(!visible_ver.end() &&
 	 visible_ver.PriorityType() &&
@@ -275,6 +307,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	return column_disposition(visible_ver.PriorityType(), 0);
       else
 	return column_disposition(_("Unknown"), 0);
+
+      break;
     case shortpriority:
       if(!visible_ver.end())
 	switch(visible_ver->Priority)
@@ -295,12 +329,14 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
       else
 	return column_disposition(_("Unknown"), 0);
 
+      break;
     case section:
       if(!visible_ver.end() && visible_ver.Section())
 	return column_disposition(visible_ver.Section(), 0);
       else
 	return column_disposition(_("Unknown"), 0);
 
+      break;
     case progname:
       return column_disposition(PACKAGE, 0);
     case progver:
@@ -316,6 +352,8 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	}
       else
 	return column_disposition("", 0);
+
+      break;
     case diskusage:
       {
 	char buf[256];
@@ -338,6 +376,7 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	  return column_disposition("", 0);
       }
 
+      break;
     case downloadsize:
       {
 	if(apt_cache_file && (*apt_cache_file)->DebSize()!=0)
@@ -351,18 +390,39 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	  return column_disposition("", 0);
       }
 
+      break;
     case autoset:
-      if((*apt_cache_file)->get_ext_state(pkg).installremove_reason!=aptitudeDepCache::manual)
+      // Display the "auto" flag UNLESS the package has been removed
+      // or purged already and is not presently being installed.
+      if((!pkg.CurrentVer().end() ||
+	  (*apt_cache_file)[pkg].Install()) &&
+	 (*apt_cache_file)->get_ext_state(pkg).install_reason!=aptitudeDepCache::manual)
 	return column_disposition("A", 0);
       else
 	return column_disposition("", 0);
 
+      break;
     case tagged:
       if((*apt_cache_file)->get_ext_state(pkg).tagged)
 	return column_disposition("*", 0);
       else
 	return column_disposition("", 0);
 
+      break;
+
+    case archive:
+      if(!visible_ver.end())
+	{
+	  string buf;
+	  for(pkgCache::VerFileIterator verfile=visible_ver.FileList(); !verfile.end(); ++verfile)
+	    {
+	      pkgCache::PkgFileIterator pkgfile=verfile.File();
+	      if(pkgfile.Archive() && strcmp(pkgfile.Archive(), "now"))
+		buf+=string(buf.empty()?"":",")+pkgfile.Archive();
+	    }
+	  return column_disposition(buf,0);
+	}
+
     case hostname:
       {
 	char buf[256];
@@ -374,6 +434,7 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	  return column_disposition(buf, 0);
       }
 
+      break;
     case revdepcount:
       {
 	int count=0;
@@ -387,11 +448,7 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	     // (I'm writing this note because I just looked at the test
 	     //  and couldn't remember what it did despite writing it
 	     //  5 minutes ago. Maybe I should have my head examined :) )
-#ifndef HAVE_LIBAPT_PKG3
-	     pkgCheckDep(D.TargetVer(), visible_ver.VerStr(), D->CompareOp))
-#else
 	     _system->VS->CheckDep(visible_ver.VerStr(), D->CompareOp, D.TargetVer()))
-#endif
 	    count++;
 
 	for(pkgCache::PrvIterator i=visible_ver.ProvidesList(); !i.end(); i++)
@@ -399,15 +456,12 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	    if(D.IsCritical() &&
 	       D->Type!=pkgCache::Dep::Conflicts &&
 	       D.ParentVer()==D.ParentPkg().CurrentVer() &&
-#ifndef HAVE_LIBAPT_PKG3
-	       pkgCheckDep(D.TargetVer(), i.ProvideVersion(), D->CompareOp))
-#else
 	       _system->VS->CheckDep(i.ProvideVersion(), D->CompareOp, D.TargetVer()))
-#endif
 	      count++;
 	snprintf(buf, 100, "%i", count);
 	return column_disposition(buf, 0);
       }
+      break;
     default:
       return column_disposition(_("ERROR"), 0);
     }
@@ -452,6 +506,8 @@ int pkg_item::pkg_columnizer::parse_column_type(char id)
       return autoset;
     case 'T':
       return tagged;
+    case 't': // like apt-get -t
+      return archive;
 
     case 'B':
       return brokencount;
@@ -485,7 +541,7 @@ public:
 void pkg_item::pkg_columnizer::init_formatting()
 {
   sscanf(_(default_widths),
-	 "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
+	 "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
 	 &defaults[name].width,
 	 &defaults[installed_size].width,
 	 &defaults[debsize].width,
@@ -502,6 +558,7 @@ void pkg_item::pkg_columnizer::init_formatting()
 	 &defaults[revdepcount].width,
 	 &defaults[autoset].width,
 	 &defaults[tagged].width,
+	 &defaults[archive].width,
 	 &defaults[brokencount].width,
 	 &defaults[diskusage].width,
 	 &defaults[downloadsize].width,
