@@ -31,7 +31,16 @@
 
 #include <map>
 
+#include <ctype.h>
+
+#include <apt-pkg/pkgsystem.h>
+#include <apt-pkg/version.h>
+
+#include <sigc++/object.h>
+#include <sigc++/object_slot.h>
+
 using namespace std;
+using namespace __gnu_cxx;
 
 // This special tree munges its tag to allow an integer to be prepended to it.
 // Ok, it's a dreadful hack.  I admit it.
@@ -151,11 +160,17 @@ void pkg_grouppolicy_section::init_section_descriptions()
   section_descriptions["doc"]=_("Documentation and specialized programs for viewing documentation\n Packages in the 'doc' section document parts of the Debian system, or are viewers for documentation formats.");
   section_descriptions["editors"]=_("Text editors and word processors\n Packages in the 'editors' section allow you to edit plain ASCII text. These are not necessarily word processors, although some word processors may be found in this section.");
   section_descriptions["electronics"]=_("Programs for working with circuits and electronics\n Packages in the 'electronics' section include circuit design tools, simulators and assemblers for microcontrollers, and other related software.");
+  section_descriptions["embedded"]=_("Programs for embedded systems\n Packages in the 'embedded' section are meant to run on embedded devices. Embedded devices are specialized hardware devices with much less power than a typical desktop system: for instance, a PDA, a cell phone, or a Tivo.");
+  section_descriptions["gnome"]=_("The GNOME Desktop System\n GNOME is a collection of software which provides an easy-to-use desktop environment for Linux.  Packages in the 'gnome' section are part of the GNOME environment or closely integrated into it.");
   section_descriptions["games"]=_("Games, toys, and fun programs\n Packages in the 'games' section are meant primarily for entertainment.");
   section_descriptions["graphics"]=_("Utilities to create, view, and edit graphics files\n Packages in the 'graphics' section include viewers for image files, image processing and manipulation software, software to interact with graphics hardware (such as video cards, scanners, and digital cameras), and programming tools for handling graphics.");
   section_descriptions["hamradio"]=_("Software for ham radio operators\n Packages in the 'hamradio' section are meant primarily for ham radio operators.");
   section_descriptions["interpreters"]=_("Interpreters for interpreted languages\n Packages in the 'interpreters' section include interpreters for languages like Python, Perl, and Ruby, and libraries for these same languages.");
+  section_descriptions["kde"]=_("The KDE Desktop System\n KDE is a collection of software which provides an easy-to-use desktop environment for Linux.  Packages in the 'kde' section are part of the KDE environment or closely integrated into it.");
+  section_descriptions["libdevel"]=_("Development files for libraries\n Packages in the 'libdevel' section contain files required for building programs that use libraries in the 'libs' section.  You don't need packages from this section unless you want to compile software yourself.");
   section_descriptions["libs"]=_("Collections of software routines\n Packages in the 'libs' section provide necessary functionality for other software on the computer. With very few exceptions, you should not need to explicitly install a package from this section; the package system will install them as required to fulfill dependencies.");
+  section_descriptions["perl"]=_("Perl interpreter and libraries\n Packages in the 'perl' section provide the Perl programming language and many third-party libraries for it. Unless you are a Perl programmer, you don't need to install packages from this section explicitly; the package system will install them if they are required.");
+  section_descriptions["python"]=_("Python interpreter and libraries\n Packages in the 'python' section provide the Python programming language and many third-party libraries for it. Unless you are a Perl programmer, you don't need to install packages from this section explicitly; the package system will install them if they are required.");
   section_descriptions["mail"]=_("Programs to write, send, and route email messages\n Packages in the 'mail' section include mail readers, mail transport daemons, mailing list software, and spam filters, as well as various other software related to electronic mail.");
   section_descriptions["math"]=_("Numeric analysis and other mathematics-related software\n Packages in the 'math' section include calculators, languages for mathematical computation (similar to Mathematica), symbolic algebra packages, and programs to visualize mathematical objects.");
   section_descriptions["misc"]=_("Miscellaneous software\n Packages in the 'misc' section have too unusual a function to be classified.");
@@ -267,10 +282,10 @@ void pkg_grouppolicy_section::add_package(const pkgCache::PkgIterator &pkg,
 
 class pkg_grouppolicy_status:public pkg_grouppolicy
 {
-  static const int numgroups=6;
+  static const int numgroups=7;
   pkg_grouppolicy_factory *chain;
 
-  enum states {newpkg, upgradable, installed, not_installed, obsolete_pkg, virtual_pkg};
+  enum states {security_upgradable, upgradable, newpkg, installed, not_installed, obsolete_pkg, virtual_pkg};
 
   static const char * const state_titles[numgroups];
   // FIXME: need better titles :)
@@ -293,15 +308,16 @@ public:
 
   virtual ~pkg_grouppolicy_status()
     {
-      for(int i=0; i<4; i++)
+      for(int i=0; i<numgroups; i++)
 	delete children[i].first;
     }
 };
 
 const char * const pkg_grouppolicy_status::state_titles[numgroups] =
 {
-  N_("New Packages\n These packages have been added to Debian since the last time you cleared the list of \"new\" packages. (choose \"Forget new packages\" from the Actions menu to empty this list)"),
+  N_("Security Updates\n Security updates for these packages are available from security.debian.org."),
   N_("Upgradable Packages\n A newer version of these packages is available."),
+  N_("New Packages\n These packages have been added to Debian since the last time you cleared the list of \"new\" packages. (choose \"Forget new packages\" from the Actions menu to empty this list)"),
   N_("Installed Packages\n These packages are currently installed on your computer."),
   N_("Not Installed Packages\n These packages are not installed on your computer."),
   N_("Obsolete and Locally Created Packages\n These packages are currently installed on your computer, but they are not available from any apt source.  They may be obsolete and removed from the archive, or you may have built a private version of them yourself."),
@@ -314,6 +330,21 @@ pkg_grouppolicy *pkg_grouppolicy_status_factory::instantiate(pkg_signal *_sig,
   return new pkg_grouppolicy_status(chain, _sig, _desc_sig);
 }
 
+// Stolen from apt-watch:
+
+/** Tests whether a particular version is security-related.
+ *
+ *  \return \b true iff the given package version comes from security.d.o
+ */
+static bool version_is_security(const pkgCache::VerIterator &ver)
+{
+  for(pkgCache::VerFileIterator F=ver.FileList(); !F.end(); ++F)
+    if(string(F.File().Site())=="security.debian.org")
+      return true;
+
+  return false;
+}
+
 void pkg_grouppolicy_status::add_package(const pkgCache::PkgIterator &pkg,
 					 pkg_subtree *root)
 {
@@ -323,8 +354,6 @@ void pkg_grouppolicy_status::add_package(const pkgCache::PkgIterator &pkg,
   if(pkg.VersionList().end())
     section=virtual_pkg;
   else
-    // Yuck.  Obsolete packages can be detected because they have only a single
-    // file entry.  Of course, libapt makes it impossible to easily check this.
     {
       if(pkg_obsolete(pkg))
 	section=obsolete_pkg;
@@ -333,7 +362,13 @@ void pkg_grouppolicy_status::add_package(const pkgCache::PkgIterator &pkg,
       else if(state.Status==2)
 	section=not_installed;
       else if(state.Upgradable())
-	section=upgradable;
+	{
+	  pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
+	  if(version_is_security(candver))
+	    section=security_upgradable;
+	  else
+	    section=upgradable;
+	}
       else
 	section=installed;
     }
@@ -387,10 +422,43 @@ bool pkg_missing_filter(const pkgCache::PkgIterator &pkg)
 
 /*****************************************************************************/
 
+/** Returns true if pkg is suggested by another package which will be
+ * installed.
+ */
+static bool package_suggested(const pkgCache::PkgIterator &pkg)
+{
+  pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
+  pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
+
+  for(pkgCache::DepIterator d=pkg.RevDependsList(); !d.end(); ++d)
+    if(d->Type==pkgCache::Dep::Suggests)
+      {
+	// Check whether the package doing the depending is going
+	// to be installed.
+	pkgCache::PkgIterator depender=d.ParentPkg();
+	pkgDepCache::StateCache &depstate=(*apt_cache_file)[depender];
+	pkgCache::VerIterator depinstver=depstate.InstVerIter(*apt_cache_file);
+
+	if(depender.CurrentVer().end() &&
+	   depstate.Install() &&
+	   !depinstver.end() &&
+	   !candver.end() &&
+	   _system->VS->CheckDep(candver.VerStr(),
+				 d->CompareOp, d.TargetVer()))
+	  {
+	    if(d.ParentVer()==depinstver)
+	      return true;
+	  }
+      }
+
+  return false;
+}
+
 class pkg_grouppolicy_mode:public pkg_grouppolicy
 {
   const static char * const child_names[];
   pair<pkg_grouppolicy *, pkg_subtree *> children[num_pkg_action_states];
+  pair<pkg_grouppolicy *, pkg_subtree *> suggested_child;
   pkg_grouppolicy_factory *chain;
 public:
   pkg_grouppolicy_mode(pkg_grouppolicy_factory *_chain,
@@ -427,6 +495,26 @@ public:
 
 	children[group].first->add_package(pkg, children[group].second);
       }
+    else if(pkg.CurrentVer().end() && package_suggested(pkg))
+      {
+	if(!suggested_child.second)
+	  {
+	    string desc=_("Packages which are suggested by other packages\n These packages are not required in order to make your system function properly, but they may provide enhanced functionality for some programs that you are currently installing.");
+	    string shortdesc=string(desc, 0, desc.find('\n'));
+
+	    pkg_subtree *newtree=new pkg_subtree_with_order(shortdesc,
+							    desc,
+							    get_desc_sig(),
+							    num_pkg_action_states,
+							    false);
+	    root->add_child(newtree);
+	    suggested_child.first=chain->instantiate(get_sig(),
+						     get_desc_sig());
+	    suggested_child.second=newtree;
+	  }
+
+	suggested_child.first->add_package(pkg, suggested_child.second);
+      }
   }
 
   virtual ~pkg_grouppolicy_mode()
@@ -438,7 +526,7 @@ public:
 
 const char * const pkg_grouppolicy_mode::child_names[num_pkg_action_states]=
 {
-  N_("Packages with unsatisfied dependencies\n The dependency requirements of these packages will be after the install is complete.\n .\n The presence of this tree probably indicates that something is broken, either on your system or in the Debian archive."),
+  N_("Packages with unsatisfied dependencies\n The dependency requirements of these packages will be unmet after the install is complete.\n .\n The presence of this tree probably indicates that something is broken, either on your system or in the Debian archive."),
   N_("Packages being removed because they are no longer used\n These packages are being deleted because they were automatically installed to fulfill dependencies, and the planned action will result in no installed package declaring an 'important' dependency on them.\n"),
   N_("Packages being automatically held in their current state\n These packages could be upgraded, but they have been kept in their current state to avoid breaking dependencies."),
   N_("Packages being automatically installed to satisfy dependencies\n These packages are being installed because they are required by another package you have chosen for installation."),
@@ -552,11 +640,7 @@ public:
 	else
 	  priority=pkg.VersionList()->Priority;
 
-#ifdef HAVE_LIBAPT_PKG3
 	pstr=(*apt_cache_file)->GetCache().Priority(priority);
-#else
-	pstr=(*apt_cache_file)->Priority(priority);
-#endif
       }
     else
       {
@@ -656,7 +740,7 @@ public:
      uncategorized_policy(NULL), chain(_chain), sig(_sig), desc_sig(_desc_sig),
      root(NULL)
   {
-    hier_reloaded.connect(SigC::slot(this, &pkg_hier_grouper::handle_reload));
+    hier_reloaded.connect(SigC::slot(*this, &pkg_hier_grouper::handle_reload));
   }
 
   ~pkg_hier_grouper()
@@ -813,6 +897,23 @@ void pkg_grouppolicy_task::init_section_descriptions()
   task_section_descriptions.push_back(pair<string,string>("unknown", "Unrecognized tasks"));
 }
 
+/** Uses the fact that 0<=Relevance<=10 to encode Relevance as a character. */
+class task_subtree:public pkg_subtree
+{
+  char relevance[2];
+public:
+  task_subtree(std::string _name, std::string _description,
+	       desc_signal *_info_signal,
+	       int _relevance)
+    :pkg_subtree(_name, _description, _info_signal)
+  {
+    relevance[0]='z'-_relevance;
+    relevance[1]=0;
+  }
+
+  const char *tag() const {return relevance;}
+};
+
 void pkg_grouppolicy_task::add_package(const pkgCache::PkgIterator &pkg,
 				       pkg_subtree *root)
 {
@@ -828,12 +929,6 @@ void pkg_grouppolicy_task::add_package(const pkgCache::PkgIterator &pkg,
 
       if(found==task_children.end())
 	{
-	  if(!tasks_subtree)
-	    {
-	      tasks_subtree=new pkg_subtree(_("Tasks"), _("\n Tasks are groups of packages which provide an easy way to select a predefined set of packages for a particular purpose."), get_desc_sig());
-	      root->add_child(tasks_subtree);
-	    }
-
 	  string section;
 	  map<string,task>::iterator taskfound=task_list->find(*i);
 	  pkg_subtree *newtree, *sectiontree;
@@ -841,7 +936,18 @@ void pkg_grouppolicy_task::add_package(const pkgCache::PkgIterator &pkg,
 	  if(taskfound==task_list->end())
 	    section="unknown";
 	  else
-	    section=taskfound->second.section;
+	    {
+	      if(!taskfound->second.keys_present())
+		continue; // skip to the next task of this package.
+
+	      section=taskfound->second.section;
+	    }
+
+	  if(!tasks_subtree)
+	    {
+	      tasks_subtree=new pkg_subtree(_("Tasks"), _("\n Tasks are groups of packages which provide an easy way to select a predefined set of packages for a particular purpose."), get_desc_sig());
+	      root->add_child(tasks_subtree);
+	    }
 
 	  subtree_map::iterator sectionfound=section_children.find(section);
 	  if(sectionfound==section_children.end())
@@ -865,11 +971,12 @@ void pkg_grouppolicy_task::add_package(const pkgCache::PkgIterator &pkg,
 	    sectiontree=sectionfound->second;
 
 	  if(taskfound!=task_list->end())
-	    newtree=new pkg_subtree(taskfound->second.shortdesc,
-				    taskfound->second.longdesc,
-				    get_desc_sig());
+	    newtree=new task_subtree(taskfound->second.shortdesc,
+				     taskfound->second.longdesc,
+				     get_desc_sig(),
+				     taskfound->second.relevance);
 	  else
-	    newtree=new pkg_subtree(*i);
+	    newtree=new task_subtree(*i, "", get_desc_sig(), 5);
 
 	  task_children[*i]=newtree;
 
