@@ -25,13 +25,15 @@
 
 #include "pkg_info_screen.h"
 
+#include "vscreen/fragment.h"
+#include "vscreen/vs_layout_item.h"
 #include "vscreen/vs_staticitem.h"
 
+#include "desc_parse.h"
 #include "pkg_subtree.h"
 #include "pkg_item_with_subtree.h"
 #include "dep_item.h"
 #include "pkg_ver_item.h"
-#include "pkg_description_treeitem.h"
 
 #include "generic/apt.h"
 
@@ -75,24 +77,53 @@ void pkg_grouppolicy_info::setup_package_info(const pkgCache::PkgIterator &pkg,
   if(!ver.end())
     {
       pkgRecords::Parser &rec=apt_package_records->Lookup(ver.FileList());
-      tree->add_child(new pkg_description(ver, _("Description")));
+
+      string desc=apt_package_records->Lookup(ver.FileList()).LongDesc();
+      string shortdesc=string(desc, 0, desc.find('\n'));
+
+      vector<fragment*> frags;
+
+      // Avoid creating new strings to translate.
+      frags.push_back(clipbox(fragf("%B%s%b%s%n",
+				    _("Description: "), shortdesc.c_str())));
+      frags.push_back(indentbox(2, 2, make_desc_fragment(desc)));
+
+      // Can I use something other than a clipbox below?
 
       if((pkg->Flags&pkgCache::Flag::Essential)==pkgCache::Flag::Essential ||
 	 (pkg->Flags&pkgCache::Flag::Important)==pkgCache::Flag::Important)
-	tree->add_child(new vs_staticitem(_("Essential: "),
-					  "yes"));
+	frags.push_back(clipbox(fragf("%B%s%b%s",
+				      _("Essential: "), _("yes"))));
 
-      tree->add_child(new vs_staticitem(_("Priority: "),pkgCache::VerIterator(ver).PriorityType()?pkgCache::VerIterator(ver).PriorityType():"Unknown"));
-      tree->add_child(new vs_staticitem(_("Section: "),pkg.Section()?pkg.Section():_("Unknown")));
-      tree->add_child(new vs_staticitem(_("Maintainer: "),rec.Maintainer()));
-      tree->add_child(new vs_staticitem(_("Compressed size: "), SizeToStr(ver->Size)));
-      tree->add_child(new vs_staticitem(_("Uncompressed size: "), SizeToStr(ver->InstalledSize)));
-      if(!rec.SourcePkg().empty())
-	tree->add_child(new vs_staticitem(_("Source Package: "),rec.SourcePkg()));
-      else
-	tree->add_child(new vs_staticitem(_("Source Package: "),pkg.Name()));
+      frags.push_back(clipbox(fragf("%B%s%b%s%n"
+				    "%B%s%b%s%n"
+				    "%B%s%b%s%n"
+				    "%B%s%b%s%n"
+				    "%B%s%b%s%n"
+				    "%B%s%b%s%n",
+				    _("Priority: "),pkgCache::VerIterator(ver).PriorityType()?pkgCache::VerIterator(ver).PriorityType():_("Unknown"),
+				    _("Section: "),pkg.Section()?pkg.Section():_("Unknown"),
+				    _("Maintainer: "),rec.Maintainer().c_str(),
+				    _("Compressed size: "), SizeToStr(ver->Size).c_str(),
+				    _("Uncompressed size: "), SizeToStr(ver->InstalledSize).c_str(),
+				    _("Source Package: "),
+				    rec.SourcePkg().empty()?pkg.Name():rec.SourcePkg().c_str())));
+
+      tree->add_child(new vs_layout_item(sequence_fragment(frags)));
 
       setup_package_deps<pkg_item_with_generic_subtree>(pkg, ver, tree, sig);
+
+      // Note: reverse provides show up in the version list
+      if(!ver.ProvidesList().end())
+	{
+	  snprintf(buf, 256, _("Package names provided by %s"), pkg.Name());
+	  pkg_subtree *prvtree=new pkg_subtree(buf);
+
+	  for(pkgCache::PrvIterator prv=ver.ProvidesList(); !prv.end(); ++prv)
+	    prvtree->add_child(new pkg_item(prv.ParentPkg(), sig));
+
+	  tree->add_child(prvtree);
+	}
     }
 
   snprintf(buf, 256, _("Packages which depend on %s"), pkg.Name());
