@@ -21,36 +21,32 @@
 
 #include "aptitude.h"
 
-#include "vscreen/config/keybindings.h"
-#include "vscreen/vs_multiplex.h"
-#include "vscreen/vs_pager.h"
-#include "vscreen/vs_util.h"
-#include "vscreen/config/column_definition.h"
+#include <vscreen/config/column_definition.h>
+#include <vscreen/config/keybindings.h>
+#include <vscreen/vs_util.h>
 
-#include "pkg_columnizer.h"
 #include "download.h"
 #include "edit_pkg_hier.h"
-#include "pkg_description_treeitem.h"
+#include "pkg_columnizer.h"
 #include "pkg_item.h"
+#include "ui.h"
+#include "view_changelog.h"
 #include "vs_progress.h"
 
-#include "download_bar.h"
-#include "download_manager.h"
-
-#include "ui.h"
-
-#include "generic/pkg_changelog.h"
-#include "generic/apt.h"
-#include "generic/matchers.h"
-#include "generic/config_signal.h"
+#include <generic/apt.h>
+#include <generic/config_signal.h>
+#include <generic/matchers.h>
 
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/error.h>
 
 #include <sigc++/bind.h>
+#include <sigc++/object_slot.h>
 
 #include <iostream>
 
+#include <unistd.h>
+
 using namespace std;
 
 static const char *confirm_str=N_("Yes, I am aware this is a very bad idea");
@@ -94,18 +90,11 @@ static void confirm_delete_essential(const pkgCache::PkgIterator &pkg,
   popup_widget(w);
 }
 
-class pkg_changelog_screen:public vs_file_pager
-// Needed to make sure the changelog is properly cleaned up.
+pkg_item::pkg_item(const pkgCache::PkgIterator &_package,
+		   SigC::Signal2<void, const pkgCache::PkgIterator &, const pkgCache::VerIterator &> *sig)
+:package(_package), info_signal(sig)
 {
-  pkg_changelog *cl;
-public:
-  pkg_changelog_screen(pkg_changelog *_cl, vscreen_widget *owner=NULL, int x=0, int y=0, int width=0, int height=0):
-    vs_file_pager(_cl->get_filename()),cl(_cl)
-  {
-  }
-
-  virtual ~pkg_changelog_screen() {delete cl;}
-};
+}
 
 void pkg_item::highlighted(vs_tree *win)
 {
@@ -199,21 +188,51 @@ void pkg_item::reinstall(undo_group *undo)
 				    undo);
 }
 
+void pkg_item::forbid_upgrade(undo_group *undo)
+{
+  pkgCache::VerIterator curver=package.CurrentVer();
+  pkgCache::VerIterator candver=(*apt_cache_file)[package].CandidateVerIter(*apt_cache_file);
+
+  if(!curver.end() && !candver.end() && curver!=candver)
+    (*apt_cache_file)->forbid_upgrade(package, candver.VerStr(), undo);
+}
+
 void pkg_item::set_auto(bool isauto, undo_group *undo)
 {
   (*apt_cache_file)->mark_auto_installed(package, isauto, undo);
 }
 
 int pkg_item::get_normal_attr()
+{
+  return pkg_attr(package, vs_treeitem::get_normal_attr());
+}
+
+int pkg_item::pkg_attr(pkgCache::PkgIterator package, int normal_attr)
 {
   if(package.VersionList().end())
     {
+      bool present_now=false, present_inst=false;
+
       for(pkgCache::PrvIterator i=package.ProvidesList(); !i.end(); i++)
 	{
 	  if(!i.OwnerPkg().CurrentVer().end())
-	    return A_BOLD|vs_treeitem::get_normal_attr();
+	    present_now=true;
+	  if(!(*apt_cache_file)[i.OwnerPkg()].InstVerIter(*apt_cache_file).end())
+	    present_inst=true;
+
+
+	  if(present_now && present_inst)
+	    break;
 	}
-      return vs_treeitem::get_normal_attr();
+
+      if(present_now && present_inst)
+	return A_BOLD|normal_attr;
+      else if(present_now)
+	return get_color("PkgToRemove");
+      else if(present_inst)
+	return get_color("PkgToInstall");
+      else
+	return normal_attr;
     }
   else
     {
@@ -233,9 +252,9 @@ int pkg_item::get_normal_attr()
       else if(state.Upgrade())
 	return get_color("PkgToUpgrade");
       else if(package.CurrentVer().end())
-	return vs_treeitem::get_normal_attr();
+	return normal_attr;
       else
-	return A_BOLD|vs_treeitem::get_normal_attr();
+	return A_BOLD|normal_attr;
     }
 }
 
@@ -279,17 +298,6 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
       vscreen_widget *w=make_dep_screen(package, visible_version(), true);
       insert_main_widget(w, buf, "");
     }
-  else if(bindings->key_matches(ch, "Description"))
-    {
-      if(!visible_version().end())
-	{
-	  char buf[512];
-	  snprintf(buf, 512, _("Description of %s"), package.Name());
-
-	  vscreen_widget *w=new pkg_description_screen(visible_version());
-	  insert_main_widget(w, buf, "");
-	}
-    }
   else if(bindings->key_matches(ch, "InfoScreen"))
     {
       char buf[512];
@@ -298,20 +306,9 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
       vscreen_widget *w=make_info_screen(package, visible_version());
       insert_main_widget(w, buf, "");
     }
-  else if(bindings->key_matches(ch, "Changelog"))
-    {
-      download_manager *widget=gen_download_progress(true);
-      pkg_changelog *cl=get_changelog(package.Name(), widget);
-      widget->Complete();
-      if(cl)
-	{
-	  char buf[512];
-	  snprintf(buf, 512, _("ChangeLog of %s"), package.Name());
-
-	  pkg_changelog_screen *newscreen=new pkg_changelog_screen(cl);
-	  insert_main_widget(newscreen, buf, "");
-	}
-    }
+  else if(bindings->key_matches(ch, "Changelog") &&
+	  !visible_version().end())
+    view_changelog(visible_version());
   else if(bindings->key_matches(ch, "InstallSingle"))
     {
       if((*apt_cache_file)[package].CandidateVerIter(*apt_cache_file).end())
@@ -319,6 +316,16 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
 
       undo_group *grp=new apt_undo_group;
       (*apt_cache_file)->mark_single_install(package, grp);
+      if(!grp->empty())
+	apt_undos->add_item(grp);
+      else
+	delete grp;
+    }
+  else if(bindings->key_matches(ch, "ForbidUpgrade"))
+    {
+      undo_group *grp=new apt_undo_group;
+      forbid_upgrade(grp);
+
       if(!grp->empty())
 	apt_undos->add_item(grp);
       else
@@ -346,7 +353,7 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
 
 	  apt_cache_file->ReleaseLock();
 
-	  printf("Reconfiguring %s\n", package.Name());
+	  printf(_("Reconfiguring %s\n"), package.Name());
 
 	  char buf[512];
 	  if(sucmd)
@@ -374,7 +381,7 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
       add_main_widget(e, _("Hierarchy editor"), "");
 
       e->connect_key("Quit", &global_bindings,
-		     slot(e, &vscreen_widget::destroy));
+		     slot(*e, &vscreen_widget::destroy));
     }
   else
     return pkg_tree_node::dispatch_char(ch, owner);
@@ -414,3 +421,23 @@ pkgCache::VerIterator pkg_item::visible_version(const pkgCache::PkgIterator &pkg
   else
     return pkg.CurrentVer();
 }
+
+const char *pkg_item::tag() const
+{
+  return package.Name();
+}
+
+const char *pkg_item::label() const
+{
+  return package.Name();
+}
+
+const pkgCache::PkgIterator &pkg_item::get_package() const
+{
+  return package;
+}
+
+pkgCache::VerIterator pkg_item::visible_version() const
+{
+  return visible_version(package);
+}
