@@ -26,8 +26,9 @@
 
 #include <apt-pkg/pkgcache.h>
 
-#include <sigc++/basic_signal.h>
+#include <sigc++/signal.h>
 
+/** A tree item which represents a package. */
 class pkg_item:public pkg_tree_node
 {
   pkgCache::PkgIterator package;
@@ -48,13 +49,12 @@ public:
   pkg_item(const pkgCache::PkgIterator &_package,
 	   SigC::Signal2<void,
 	   const pkgCache::PkgIterator &,
-	   const pkgCache::VerIterator &> *sig)
-    :package(_package), info_signal(sig) {}
+	   const pkgCache::VerIterator &> *sig);
 
   virtual void paint(vs_tree *win, int y, bool hierarchical);
 
-  virtual const char *tag() const {return package.Name();}
-  virtual const char *label() const {return package.Name();}
+  virtual const char *tag() const;
+  virtual const char *label() const;
   virtual bool matches(std::string s) const;
 
   virtual void select(undo_group *undo);
@@ -63,18 +63,24 @@ public:
   virtual void purge(undo_group *undo);
   virtual void reinstall(undo_group *undo);
   virtual void set_auto(bool isauto, undo_group *undo);
+  virtual void forbid_upgrade(undo_group *undo);
 
   virtual void highlighted(vs_tree *win);
   virtual void unhighlighted(vs_tree *win);
 
   virtual int get_normal_attr();
 
-  const pkgCache::PkgIterator &get_package() const {return package;}
-  pkgCache::VerIterator visible_version() const
-  {return visible_version(package);}
+  const pkgCache::PkgIterator &get_package() const;
+  pkgCache::VerIterator visible_version() const;
 
   bool dispatch_char(chtype ch, vs_tree *owner);
   void dispatch_mouse(short id, int x, mmask_t bstate, vs_tree *owner);
+
+  /** Returns the attributes that would be used to display the given
+   *  package.  normal_attr indicates the "background" attributes that
+   *  should be used.
+   */
+  static int pkg_attr(pkgCache::PkgIterator package, int normal_attr);
 };
 
 #endif
