@@ -47,6 +47,7 @@
 #include <apt-pkg/algorithms.h>
 
 #include <sigc++/bind.h>
+#include <sigc++/object_slot.h>
 
 keybindings *pkg_tree::bindings=NULL;
 
@@ -80,17 +81,11 @@ void pkg_tree::redirect_cancel_incsearch(string s)
   do_cancel_incsearch();
 }
 
-pkg_tree::pkg_tree(const char *def_grouping,
-		   pkg_grouppolicy_factory *_grouping,
-		   const char *def_limit):
-  grouping(_grouping), groupingstr(def_grouping),
-  sorting(parse_sortpolicy(aptcfg->Find(PACKAGE "::UI::Default-Sorting",
-					 "name"))),
-  limit(NULL), last_search_matcher(NULL),
-  doing_incsearch(false), pre_incsearch_selected(get_end())
+void pkg_tree::init(const char *def_limit,
+		    OpProgress &progress)
 {
   aptcfg->connect(PACKAGE "::UI::Incremental-Search",
-		  SigC::slot(this, &pkg_tree::redirect_cancel_incsearch));
+		  SigC::slot(*this, &pkg_tree::redirect_cancel_incsearch));
 
   if(def_limit)
     limitstr=def_limit;
@@ -101,11 +96,38 @@ pkg_tree::pkg_tree(const char *def_grouping,
     limit=parse_pattern(limitstr);
 
   if(grouping)
-    build_tree();
+    build_tree(progress);
 
-  cache_closed.connect(slot(this, &pkg_tree::handle_cache_close));
+  cache_closed.connect(slot(*this, &pkg_tree::handle_cache_close));
 
-  cache_reloaded.connect(slot(this, &pkg_tree::build_tree));
+  cache_reloaded.connect(SigC::slot<void, pkg_tree, pkg_tree>(*this, &pkg_tree::build_tree));
+}
+
+pkg_tree::pkg_tree(const char *def_grouping,
+		   pkg_grouppolicy_factory *_grouping,
+		   const char *def_limit,
+		   OpProgress &progress):
+  grouping(_grouping), groupingstr(def_grouping),
+  sorting(parse_sortpolicy(aptcfg->Find(PACKAGE "::UI::Default-Sorting",
+					 "name"))),
+  limit(NULL), last_search_matcher(NULL),
+  doing_incsearch(false), pre_incsearch_selected(get_end())
+{
+  init(def_limit, progress);
+}
+
+pkg_tree::pkg_tree(const char *def_grouping,
+		   pkg_grouppolicy_factory *_grouping,
+		   const char *def_limit):
+  grouping(_grouping), groupingstr(def_grouping),
+  sorting(parse_sortpolicy(aptcfg->Find(PACKAGE "::UI::Default-Sorting",
+					 "name"))),
+  limit(NULL), last_search_matcher(NULL),
+  doing_incsearch(false), pre_incsearch_selected(get_end())
+{
+  OpProgress p;
+
+  init(def_limit, p);
 }
 
 void pkg_tree::handle_cache_close()
@@ -159,7 +181,7 @@ void pkg_tree::set_sorting(string s)
     set_sorting(policy);
 }
 
-void pkg_tree::build_tree()
+void pkg_tree::build_tree(OpProgress &progress)
 {
   doing_incsearch=false;
 
@@ -176,9 +198,18 @@ void pkg_tree::build_tree()
 
       mytree->set_depth(-1);
 
+      int num=0;
+      int total=(*apt_cache_file)->Head().PackageCount;
+
       for(pkgCache::PkgIterator i=(*apt_cache_file)->PkgBegin(); !i.end(); i++)
-	if((!limit) || limit->matches(i, pkg_item::visible_version(i)))
-	  grouper->add_package(i, mytree);
+	{
+	  progress.OverallProgress(num, total, 1, _("Building view"));
+
+	  if((!limit) || limit->matches(i, pkg_item::visible_version(i)))
+	    grouper->add_package(i, mytree);
+	}
+
+      progress.OverallProgress(total, total, 1, _("Building view"));
 
       mytree->sort(pkg_sortpolicy_wrapper(sorting));
 
@@ -190,6 +221,13 @@ void pkg_tree::build_tree()
   vscreen_update();
 }
 
+void pkg_tree::build_tree()
+{
+  vs_progress *p=gen_progress_bar();
+  build_tree(*p);
+  p->destroy();
+}
+
 void pkg_tree::set_limit(string _limit)
 {
   if(limit)
@@ -209,14 +247,14 @@ bool pkg_tree::handle_char(chtype ch)
   if(bindings->key_matches(ch, "ChangePkgTreeLimit"))
     prompt_string(_("Enter the new package tree limit: "),
 		  limitstr,
-		  SigC::slot(this, &pkg_tree::set_limit),
+		  SigC::slot(*this, &pkg_tree::set_limit),
 		  NULL,
 		  NULL,
 		  &limit_history);
   else if(bindings->key_matches(ch, "ChangePkgTreeGrouping"))
     prompt_string(_("Enter the new package grouping mechanism for this display: "),
 		  groupingstr,
-		  SigC::slot(this,
+		  SigC::slot(*this,
 			     (void (pkg_tree::*) (string)) &pkg_tree::set_grouping),
 		  NULL,
 		  NULL,
@@ -224,7 +262,7 @@ bool pkg_tree::handle_char(chtype ch)
   else if(bindings->key_matches(ch, "ChangePkgTreeSorting"))
     prompt_string(_("Enter the new package sorting mechanism for this display: "),
 		  "",
-		  SigC::slot(this,
+		  SigC::slot(*this,
 			     (void (pkg_tree::*) (string)) &pkg_tree::set_sorting),
 		  NULL,
 		  NULL,
@@ -232,9 +270,9 @@ bool pkg_tree::handle_char(chtype ch)
   else if(bindings->key_matches(ch, "Search"))
     prompt_string(_("Search for: "),
 		  "",
-		  SigC::slot(this, &pkg_tree::do_search),
-		  SigC::slot(this, &pkg_tree::do_cancel_incsearch),
-		  SigC::slot(this, &pkg_tree::do_incsearch),
+		  SigC::slot(*this, &pkg_tree::do_search),
+		  SigC::slot(*this, &pkg_tree::do_cancel_incsearch),
+		  SigC::slot(*this, &pkg_tree::do_incsearch),
 		  &search_history);
   else if(bindings->key_matches(ch, "ReSearch"))
     {
