@@ -29,6 +29,8 @@
 
 #include "generic/matchers.h"
 
+class OpProgress;
+
 class pkg_grouppolicy_factory;
 
 class pkg_sortpolicy;
@@ -62,19 +64,33 @@ class pkg_tree:public vs_tree
   void do_cancel_incsearch();
   void redirect_cancel_incsearch(std::string s);
   void handle_cache_close();
+
+  void init(const char *limitstr,
+	    OpProgress &progress);
 protected:
   virtual bool handle_char(chtype ch);
 public:
+  pkg_tree(const char *groupingstr,
+	   pkg_grouppolicy_factory *_grouping,
+	   const char *limitstr,
+	   OpProgress &progress);
+
   pkg_tree(const char *groupingstr,
 	   pkg_grouppolicy_factory *_grouping,
 	   const char *limitstr);
+
   // Should you be able to just pass in a string (?)
   ~pkg_tree();
 
-  void build_tree();
+  void build_tree(OpProgress &progress);
   // [re]builds the tree using the current grouping strategy.
   // (also sorts it)
 
+  /** Rebuilds the tree using the current grouping policy.  Creates a
+   *  progress bar using gen_progress_bar().
+   */
+  void build_tree();
+
   void set_grouping(pkg_grouppolicy_factory *_grouping);
   void set_grouping(std::string s);
   void set_sorting(pkg_sortpolicy *_sorting);
