@@ -21,16 +21,14 @@
 
 #include "aptitude.h"
 
-#include "pkg_item.h"
+#include "dep_item.h"
 #include "pkg_columnizer.h"
+#include "pkg_info_screen.h"
+#include "pkg_item.h"
 #include "pkg_subtree.h"
 #include "pkg_ver_item.h"
-
-#include "pkg_description_treeitem.h"
-#include "dep_item.h"
-#include "pkg_info_screen.h"
-
 #include "ui.h"
+#include "view_changelog.h"
 
 #include "generic/apt.h"
 #include "generic/config_signal.h"
@@ -72,6 +70,22 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	return column_disposition(string(ver.ParentPkg().Name())+" "+ver.VerStr(), basex);
       else
 	return column_disposition(ver.VerStr(), basex);
+
+      break;
+
+    case archive:
+      if(!ver.end())
+	{
+	  string buf;
+	  for(pkgCache::VerFileIterator verfile=ver.FileList(); !verfile.end(); ++verfile)
+	    {
+	      pkgCache::PkgFileIterator pkgfile=verfile.File();
+	      if(pkgfile.Archive() && strcmp(pkgfile.Archive(), "now"))
+		buf+=string(buf.empty()?"":",")+pkgfile.Archive();
+	    }
+	  return column_disposition(buf,0);
+	}
+
     case installed_size:
       if(ver.end())
 	return column_disposition("", 0);
@@ -79,6 +93,8 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	return column_disposition(SizeToStr(ver->InstalledSize)+'B', 0);
       else
 	return column_disposition(_("<N/A>"), 0);
+
+      break;
     case debsize:
       if(ver.end())
 	return column_disposition("", 0);
@@ -122,6 +138,8 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	default:
 	  return column_disposition("E", 0);
 	}
+
+      break;
     case longstate:
       if(ver.end())
 	return column_disposition("", 0);
@@ -152,21 +170,29 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	default:
 	  return column_disposition(_("ERROR"), 0);
 	}
+
+      break;
     case actionflag:
       {
 	if(ver.end())
 	  return column_disposition("", 0);
 
-	aptitudeDepCache::StateCache state=(*apt_cache_file)[ver.ParentPkg()];
-	if(state.Status!=2 && (*apt_cache_file)->get_ext_state(ver.ParentPkg()).selection_state==pkgCache::State::Hold && !state.NowBroken())
+	pkgCache::PkgIterator pkg=ver.ParentPkg();
+	aptitudeDepCache::StateCache &state=(*apt_cache_file)[pkg];
+	aptitudeDepCache::aptitude_state &estate=(*apt_cache_file)->get_ext_state(pkg);
+	pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
+
+	if(state.Status!=2 && estate.selection_state==pkgCache::State::Hold && !state.NowBroken())
 	  return column_disposition("h", 0);
+	else if(ver.VerStr() == estate.forbidver)
+	  return column_disposition("F", 0);
 	else if(state.Delete())
 	  return column_disposition((state.iFlags&pkgDepCache::Purge)?"p":"d", 0);
-	else if(state.InstBroken() && (*apt_cache_file)[ver.ParentPkg()].InstVerIter(*apt_cache_file)==ver)
+	else if(state.InstBroken() && state.InstVerIter(*apt_cache_file)==ver)
 	  return column_disposition("B", 0);
 	else if(state.NewInstall())
 	  {
-	    if((*apt_cache_file)[ver.ParentPkg()].CandidateVerIter(*apt_cache_file)==ver)
+	    if(candver==ver)
 	      return column_disposition("i", 0);
 	    else
 	      return column_disposition(" ", 0);
@@ -182,27 +208,34 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	  {
 	    if(ver.ParentPkg().CurrentVer()==ver)
 	      return column_disposition("d", 0);
-	    else if((*apt_cache_file)[ver.ParentPkg()].CandidateVerIter(*apt_cache_file)==ver)
+	    else if(candver==ver)
 	      return column_disposition("i", 0);
+	    else
+	      return column_disposition(" ", 0);
 	  }
 	else
 	  return column_disposition(" ", 0);
       }
+
+      break;
     case longaction:
       {
 	if(ver.end())
 	  return column_disposition("", 0);
 
 	aptitudeDepCache::StateCache state=(*apt_cache_file)[ver.ParentPkg()];
-	if(state.Status!=2 && (state.Held() || (*apt_cache_file)->get_ext_state(ver.ParentPkg()).selection_state==pkgCache::State::Hold) && !state.NowBroken())
+	aptitudeDepCache::aptitude_state &estate=(*apt_cache_file)->get_ext_state(ver.ParentPkg());
+	if(state.Status!=2 && (state.Held() || estate.selection_state==pkgCache::State::Hold) && !state.NowBroken())
 	  return column_disposition(_("hold"), 0);
+	else if(ver.VerStr() == estate.forbidver)
+	  return column_disposition("forbidden version", 0);
 	else if(state.Delete())
 	  return column_disposition((state.iFlags&pkgDepCache::Purge)?"p":"d", 0);
-	else if(state.InstBroken() && (*apt_cache_file)[ver.ParentPkg()].InstVerIter(*apt_cache_file)==ver)
+	else if(state.InstBroken() && state.InstVerIter(*apt_cache_file)==ver)
 	  return column_disposition(_("broken"), 0);
 	else if(state.NewInstall())
 	  {
-	    if((*apt_cache_file)[ver.ParentPkg()].CandidateVerIter(*apt_cache_file)==ver)
+	    if(state.CandidateVerIter(*apt_cache_file)==ver)
 	      return column_disposition(_("install"), 0);
 	    else
 	      return column_disposition(_("none"), 0);
@@ -220,10 +253,14 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	      return column_disposition(_("delete"), 0);
 	    else if((*apt_cache_file)[ver.ParentPkg()].CandidateVerIter(*apt_cache_file)==ver)
 	      return column_disposition(_("install"), 0);
+	    else
+	      return column_disposition(_("none"), 0);
 	  }
 	else
 	  return column_disposition(_("none"), 0);
       }
+
+      break;
     case description:
       if(ver.end())
 	return column_disposition("", 0);
@@ -261,6 +298,8 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	default:
 	  return column_disposition(_("ERR"), 0);
 	}
+
+      break;
     case section:
       if(ver.end())
 	return column_disposition("", 0);
@@ -269,6 +308,8 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	return column_disposition(ver.Section(), 0);
       else
 	return column_disposition(_("Unknown"), 0);
+
+      break;
     case revdepcount:
       {
 	if(ver.end())
@@ -286,20 +327,12 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	      //  5 minutes ago. Maybe I should have my head examined :) )
 	      {
 		if((i->CompareOp&~pkgCache::Dep::Or)==pkgCache::Dep::NoOp ||
-#ifndef HAVE_LIBAPT_PKG3
-		   pkgCheckDep(i.TargetVer(), ver.VerStr(), i->CompareOp))
-#else
 		   _system->VS->CheckDep(ver.VerStr(), i->CompareOp, i.TargetVer()))
-#endif
 		  count++;
 
 		for(pkgCache::PrvIterator j=ver.ProvidesList(); !j.end(); j++)
 		  if((i->CompareOp&~pkgCache::Dep::Or)==pkgCache::Dep::NoOp ||
-#ifndef HAVE_LIBAPT_PKG3
-		     pkgCheckDep(i.TargetVer(), j.ProvideVersion(), i->CompareOp))
-#else
 		     _system->VS->CheckDep(j.ProvideVersion(), i->CompareOp, i.TargetVer()))
-#endif
 		    count++;
 	      }
 	  }
@@ -355,6 +388,11 @@ pkg_grouppolicy *pkg_grouppolicy_ver_factory::instantiate(pkg_signal *sig,
 }
 
 int pkg_ver_item::get_normal_attr()
+{
+  return ver_attr(version, vs_treeitem::get_normal_attr());
+}
+
+int pkg_ver_item::ver_attr(pkgCache::VerIterator version, int normal_attr)
 {
   pkgCache::PkgIterator pkg=version.ParentPkg();
   pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
@@ -377,13 +415,13 @@ int pkg_ver_item::get_normal_attr()
     return get_color("PkgBroken");
 
   else if(pkg.CurrentVer()!=version)
-    return vs_treeitem::get_normal_attr();
+    return normal_attr;
 
   else if(state.NowBroken())
     return get_color("PkgBroken");
 
   else
-    return A_BOLD|vs_treeitem::get_normal_attr();
+    return A_BOLD|normal_attr;
 }
 
 void pkg_ver_item::paint(vs_tree *win, int y, bool hierarchical)
@@ -439,7 +477,6 @@ void pkg_ver_item::select(undo_group *undo)
 	  (*apt_cache_file)->get_ext_state(version.ParentPkg()).selection_state=pkgCache::State::Install;
 	}
     }
-#ifdef HAVE_LIBAPT_PKG3
   else
     {
       undo_group *grp=undo?new undo_group:NULL;
@@ -449,12 +486,6 @@ void pkg_ver_item::select(undo_group *undo)
       if(undo)
 	undo->add_item(grp);
     }
-#else
-  else if(version==(*apt_cache_file)[version.ParentPkg()].CandidateVerIter(*apt_cache_file))
-    (*apt_cache_file)->mark_install(version.ParentPkg(), aptcfg->FindB(PACKAGE "::Auto-Install", true), false, undo);
-  // FIXME: this should work even for other versions of the package (besides
-  //       the 'target' and current versions)
-#endif
 }
 
 void pkg_ver_item::remove(undo_group *undo)
@@ -539,21 +570,17 @@ bool pkg_ver_item::dispatch_char(chtype ch, vs_tree *owner)
       insert_main_widget(w, buf, "");
       return true;
     }
-  else if(bindings->key_matches(ch, "Description"))
-    {
-      char buf[512];
-      snprintf(buf, 512, _("Description of %s"), version.ParentPkg().Name());
-
-      vscreen_widget *w=new pkg_description_screen(version);
-      insert_main_widget(w, string(_("Description of "))+version.ParentPkg().Name(), "");
-      return true;
-    }
   else if(bindings->key_matches(ch, "InfoScreen"))
     {
       vscreen_widget *w=make_info_screen(version.ParentPkg(), version);
       insert_main_widget(w, string(_("Information about "))+version.ParentPkg().Name(), "");
       return true;
     }
+  else if(bindings->key_matches(ch, "Changelog"))
+    {
+      view_changelog(version);
+      return true;
+    }
   else
     return pkg_tree_node::dispatch_char(ch, owner);
 }
