@@ -32,9 +32,7 @@
 
 #include <apt-pkg/version.h>
 #include <apt-pkg/depcache.h>
-#ifdef HAVE_LIBAPT_PKG3
 #include <apt-pkg/pkgsystem.h>
-#endif
 
 class pkg_ver_item:public pkg_tree_node
 {
@@ -69,6 +67,8 @@ public:
   bool dispatch_char(chtype ch, vs_tree *owner);
 
   pkg_ver_item *get_sig();
+
+  static int ver_attr(pkgCache::VerIterator version, int normal_attr);
 };
 
 class versort:public sortpolicy
@@ -80,11 +80,7 @@ public:
       return strcmp(item1->get_version().ParentPkg().Name(),
 		    item2->get_version().ParentPkg().Name());
 
-#ifndef HAVE_LIBAPT_PKG3
-    return pkgVersionCompare(item1->get_version().VerStr(), item2->get_version().VerStr());
-#else
     return _system->VS->CmpVersion(item1->get_version().VerStr(), item2->get_version().VerStr());
-#endif
   }
 
   bool operator()(const vs_treeitem *item1, const vs_treeitem *item2) const
