@@ -7,6 +7,7 @@
 #include "ui.h"
 
 #include <sigc++/bind.h>
+#include <sigc++/object_slot.h>
 
 #include <apt-pkg/acquire.h>
 #include <apt-pkg/clean.h>
@@ -18,16 +19,19 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 
+#include <utility>
+
 #include "../config.h"
 #include "aptitude.h"
 
+#include "apt_options.h"
+#include "defaults.h"
 #include "load_config.h"
 #include "load_grouppolicy.h"
 #include "load_pkgview.h"
-#include "defaults.h"
-#include "apt_options.h"
 
 #include <vscreen/curses++.h>
+#include <vscreen/fragment.h>
 #include <vscreen/vscreen.h>
 #include <vscreen/vs_button.h>
 #include <vscreen/vs_center.h>
@@ -41,6 +45,7 @@
 #include <vscreen/vs_stacked.h>
 #include <vscreen/vs_statuschoice.h>
 #include <vscreen/vs_table.h>
+#include <vscreen/vs_text_layout.h>
 #include <vscreen/vs_tree.h>
 #include <vscreen/vs_util.h>
 
@@ -49,8 +54,6 @@
 #include <generic/apt.h>
 #include <generic/config_signal.h>
 
-#include <sigc++/convert.h>
-
 #include "dep_item.h"
 #include "download.h"
 #include "download_screen.h"
@@ -65,6 +68,8 @@
 #include "pkg_view.h"
 #include "vs_progress.h"
 
+using namespace std;
+
 static vs_menubar *main_menu;
 static vs_menu *views_menu;
 
@@ -92,59 +97,111 @@ static bool active_download;
 vscreen_widget *active_status_download;
 
 SigC::Signal0<void> file_quit;
+SigC::Signal0<void> package_states_changed;
 
 const char *default_pkgstatusdisplay="%d";
 const char *default_pkgheaderdisplay="%N %n #%B %u %o";
 const char *default_grpstr="filter(missing),task,status,section(subdir,passthrough),section(topdir)";
 
-class apt_error_pager:public vs_pager
+class apt_error_pager:public vs_text_layout
 {
-  string text;
+  unsigned int maxlen;
 public:
-  apt_error_pager():vs_pager(NULL)
+  apt_error_pager():vs_text_layout(), maxlen(0)
   {
+    vector<fragment *> frags;
+
     if(_error->empty())
-      text=_("Er, there aren't any errors, this shouldn't have happened..");
+      frags.push_back(text_fragment(_("Er, there aren't any errors, this shouldn't have happened..")));
     else while(!_error->empty())
       {
-	string currerr;
+	string currerr, tag;
 	bool iserr=_error->PopMessage(currerr);
 	if(iserr)
-	  text=text+"E: "+currerr+'\n';
+	  tag="E:";
 	else
-	  text=text+"W: "+currerr+'\n';
+	  tag="W:";
+
+	maxlen=max(maxlen, tag.size()+1+currerr.size());
+
+	frags.push_back(fragf("%s %s", tag.c_str(), currerr.c_str()));
       }
 
-    set_text(text.c_str());
+    set_fragment(attr_fragment(indentbox(0, 3,
+					 wrapbox(sequence_fragment(frags))),
+			       get_color("Error")));
+  }
+
+  // Try to get a large enough size to display all errors with or
+  // without wrapping.
+  size size_request()
+  {
+    // HACK: I know the frame takes up a certain X space, so...
+    //
+    // The long-term solution is to decouple X and Y layout (ie, allocate
+    // space in the X dimension in one step, then allocate space in the Y
+    // direction once X has been finalized).  However, this takes a lot of
+    // effort and testing and is not suitable for the current aptitude
+    // release (which hopefully will be what goes into sarge)
+    unsigned int top_w=rootwin.getmaxx();
+
+    // Allow 2 characters for the frame and 1 for the scrollbar.
+    if(top_w<3)
+      top_w=0;
+    else
+      top_w-=3;
+
+    // See how many lines we need to display things at the size of
+    // the largest widget we know about.  Add extra padding to the length
+    // because of the indentbox's behavior.
+    return size(min(top_w, maxlen+3), height_for_width(top_w));
   }
 };
 
+// Search callback
+static void pager_do_search(std::string s, vs_pager *p)
+{
+  p->search_for(s);
+}
+
+// Handles "search" dialogs for pagers
+static void pager_search(vs_pager *p)
+{
+  prompt_string(_("Search for:"),
+		p->get_last_search(),
+		bind(slot(pager_do_search), p),
+		NULL,
+		NULL,
+		NULL);
+}
+
 void check_apt_errors()
 {
   if(!_error->empty())
     {
       vs_label *l=new vs_label("Apt errors");
       l->set_bg(get_color("Error"));
-      vs_pager *p=new apt_error_pager;
+      apt_error_pager *p=new apt_error_pager;
       p->set_bg(get_color("Error"));
       vs_button *b=new vs_button("Ok");
       vs_table *t=new vs_table;
       t->set_bg(get_color("Error"));
       vs_center *c=new vs_center(new vs_frame(t));
-      vs_scrollbar *scrl=new vs_scrollbar(vs_scrollbar::VERTICAL,
-					  p->get_cury(),
-					  p->get_maxy());
-      b->pressed.connect(slot(c, &vs_center::destroy));
+
+      vs_scrollbar *scrl=new vs_scrollbar(vs_scrollbar::VERTICAL);
+      p->location_changed.connect(slot(*scrl, &vs_scrollbar::set_slider));
+      scrl->scrollbar_interaction.connect(slot(*p, &vs_text_layout::scroll));
+
+      b->pressed.connect(slot(*c, &vs_center::destroy));
       t->add_widget(l, 0, 0, 1, 2, false);
       t->add_widget_opts(p, 1, 0, 1, 1,
-			 vs_table::ALIGN_CENTER | vs_table::SHRINK,
-			 vs_table::ALIGN_CENTER | vs_table::SHRINK);
+			 vs_table::EXPAND,
+			 vs_table::EXPAND);
       t->add_widget(scrl, 1, 1, 1, 1,
 		    vs_table::ALIGN_CENTER,
-		    vs_table::ALIGN_CENTER | vs_table::EXPAND | vs_table::SHRINK);
+		    vs_table::ALIGN_CENTER | vs_table::EXPAND);
       t->add_widget(new vs_center(b), 2, 0, 1, 2, false);
       t->connect_key("Confirm", &global_bindings, b->pressed.slot());
-      p->location_changed.connect(slot(scrl, &vs_scrollbar::set_slider));
 
       main_stacked->add_visible_widget(c, true);
     }
@@ -314,7 +371,7 @@ static void do_destroy_visible()
       active_status_download=NULL;
     }
   else if(aptcfg->FindB(PACKAGE "::UI::Exit-On-Last-Close", true) &&
-     main_multiplex->num_children()==1)
+     main_multiplex->num_children()<=1)
     do_quit();
   else
     {
@@ -374,7 +431,8 @@ static void do_revert_options()
 static vscreen_widget *make_default_view(vscreen_widget *mainwidget,
 					 pkg_signal *sig,
 					 desc_signal *desc_sig,
-					 bool allow_visible_desc=true)
+					 bool allow_visible_desc=true,
+					 bool show_reason_first=false)
 {
   if(aptcfg->Exists(PACKAGE "::UI::Default-Package-View"))
     {
@@ -383,7 +441,7 @@ static vscreen_widget *make_default_view(vscreen_widget *mainwidget,
       if(format)
 	{
 	  vscreen_widget *rval=make_package_view(*format, mainwidget, sig,
-						 desc_sig);
+						 desc_sig, show_reason_first);
 	  delete format;
 
 	  if(rval)
@@ -432,17 +490,18 @@ static vscreen_widget *make_default_view(vscreen_widget *mainwidget,
 					   "ShowHideDescription", "",
 					   allow_visible_desc && aptcfg->FindB(PACKAGE "::UI::Description-Visible-By-Default", true)));
 
-  return make_package_view(basic_format, mainwidget, sig, desc_sig);
+  return make_package_view(basic_format, mainwidget, sig, desc_sig,
+			   show_reason_first);
 }
 
-void do_new_package_view()
+void do_new_package_view(OpProgress &progress)
 {
   pkg_grouppolicy_factory *grp=NULL;
   std::string grpstr="";
 
-  if(aptcfg->Exists("Aptitude::UI::Default-Grouping"))
+  if(aptcfg->Exists(PACKAGE "::UI::Default-Grouping"))
     {
-      grpstr=aptcfg->Find("Aptitude::UI::Default-Grouping");
+      grpstr=aptcfg->Find(PACKAGE "::UI::Default-Grouping");
       grp=parse_grouppolicy(grpstr);
     }
 
@@ -457,16 +516,24 @@ void do_new_package_view()
 	grp=new pkg_grouppolicy_filter_factory(pkg_missing_filter,new pkg_grouppolicy_task_factory(new pkg_grouppolicy_status_factory(new pkg_grouppolicy_section_factory(pkg_grouppolicy_section_factory::split_subdir,true,new pkg_grouppolicy_section_factory(pkg_grouppolicy_section_factory::split_topdir,false,new pkg_grouppolicy_end_factory())))));
     }
 
-  pkg_tree *tree=new pkg_tree(grpstr.c_str(), grp, NULL);
+  pkg_tree *tree=new pkg_tree(grpstr.c_str(), grp, NULL, progress);
 
   add_main_widget(make_default_view(tree,
 				    &tree->selected_signal,
 				    &tree->selected_desc_signal),
-		  "Packages",
+		  _("Packages"),
 		  _("View available packages and choose actions to perform"));
 }
 
-void do_new_hier_view()
+// For signal connections.
+static void do_new_package_view_with_new_bar()
+{
+  vs_progress *p=gen_progress_bar();
+  do_new_package_view(*p);
+  p->destroy();
+}
+
+void do_new_hier_view(OpProgress &progress)
 {
   pkg_grouppolicy_factory *grp=NULL;
   std::string grpstr="";
@@ -474,7 +541,7 @@ void do_new_hier_view()
   grpstr="filter(missing),hier";
   grp=parse_grouppolicy(grpstr);
 
-  pkg_tree *tree=new pkg_tree(grpstr.c_str(), grp, NULL);
+  pkg_tree *tree=new pkg_tree(grpstr.c_str(), grp, NULL, progress);
   tree->set_limit("!~v");
   tree->set_hierarchical(false);
 
@@ -485,6 +552,14 @@ void do_new_hier_view()
 		  _("View available packages and choose actions to perform"));
 }
 
+// For signal connections.
+static void do_new_hier_view_with_new_bar()
+{
+  vs_progress *p=gen_progress_bar();
+  do_new_hier_view(*p);
+  p->destroy();
+}
+
 vscreen_widget *make_info_screen(const pkgCache::PkgIterator &pkg,
 				 const pkgCache::VerIterator &ver)
 {
@@ -497,13 +572,13 @@ vscreen_widget *make_dep_screen(const pkgCache::PkgIterator &pkg,
 				bool reverse)
 {
   pkg_dep_screen *w=new pkg_dep_screen(pkg, ver, reverse);
-  return make_default_view(w, w->get_sig(), w->get_desc_sig(), false);
+  return make_default_view(w, w->get_sig(), w->get_desc_sig(), true);
 }
 
 vscreen_widget *make_ver_screen(const pkgCache::PkgIterator &pkg)
 {
   pkg_ver_screen *w=new pkg_ver_screen(pkg);
-  return make_default_view(w, w->get_sig(), w->get_desc_sig(), false);
+  return make_default_view(w, w->get_sig(), w->get_desc_sig(), true);
 }
 
 //static void silly_test_error(string s)
@@ -525,7 +600,8 @@ static void do_help_about()
 
 static void do_help_license()
 {
-  vscreen_widget *w=vs_dialog_fileview(HELPDIR "/COPYING");
+  vscreen_widget *w=vs_dialog_fileview(HELPDIR "/COPYING",
+				       NULL, slot(pager_search));
   w->show_all();
 
   popup_widget(w);
@@ -537,7 +613,7 @@ static void do_help_help()
 
   snprintf(buf, 512, HELPDIR "/%s", _("help.txt"));
 
-  vscreen_widget *w=vs_dialog_fileview(buf);
+  vscreen_widget *w=vs_dialog_fileview(buf, NULL, slot(pager_search));
   w->show_all();
 
   popup_widget(w);
@@ -549,15 +625,23 @@ static void do_help_readme()
 
   snprintf(buf, 512, HELPDIR "/%s", _("README")); // README can be translated..
 
-  vscreen_widget *w=vs_dialog_fileview(buf);
+  vscreen_widget *w=vs_dialog_fileview(buf, NULL, slot(pager_search));
   w->show_all();
   popup_widget(w);
 }
 
+static void do_help_faq()
+{
+  vscreen_widget *w=vs_dialog_fileview(HELPDIR "/FAQ", NULL, slot(pager_search));
+  w->show_all();
+
+  popup_widget(w);
+}
+
 // news isn't translated since it's just a changelog.
 static void do_help_news()
 {
-  vscreen_widget *w=vs_dialog_fileview(HELPDIR "/NEWS");
+  vscreen_widget *w=vs_dialog_fileview(HELPDIR "/NEWS", NULL, slot(pager_search));
   w->show_all();
 
   popup_widget(w);
@@ -607,7 +691,7 @@ static void do_show_preview()
 
       if(aptcfg->Exists(PACKAGE "::UI::Default-Preview-Grouping"))
 	{
-	  grpstr=aptcfg->Find("Aptitude::UI::Default-Preview-Grouping");
+	  grpstr=aptcfg->Find(PACKAGE "::UI::Default-Preview-Grouping");
 	  grp=parse_grouppolicy(grpstr);
 	}
 
@@ -630,10 +714,12 @@ static void do_show_preview()
 
       active_preview=make_default_view(active_preview_tree,
 				       &active_preview_tree->selected_signal,
-				       &active_preview_tree->selected_desc_signal);
+				       &active_preview_tree->selected_desc_signal,
+				       true,
+				       true);
 
       active_preview->destroyed.connect(SigC::slot(reset_preview));
-      active_preview_tree->destroyed.connect(SigC::slot(active_preview, &vscreen_widget::destroy));
+      active_preview_tree->destroyed.connect(SigC::slot(*active_preview, &vscreen_widget::destroy));
       active_preview_tree->connect_key("DoInstallRun",
 				       &global_bindings,
 				       SigC::slot(actually_do_package_run));
@@ -661,7 +747,7 @@ static void actually_do_package_run()
 	{
 	  if((*apt_cache_file)->BrokenCount()>0)
 	    {
-	      if(_config->FindB("Aptitude::Auto-Fix-Broken", true))
+	      if(_config->FindB(PACKAGE "::Auto-Fix-Broken", true))
 		{
 		  undo_group *undo=new undo_group;
 		  (*apt_cache_file)->try_fix_broken(undo);
@@ -704,9 +790,9 @@ static void actually_do_package_run()
 	  popup_widget(vs_dialog_yesno(_("Installing/removing packages requires administrative privileges, which\nyou currently do not have.  Would you like to change to the root account?\n"),
 				       bind(SigC::slot(&do_su_to_root),
 					    "-i"),
-				       "Become root",
+				       _("Become root"),
 				       SigC::slot(&install_or_remove_packages),
-				       "Don't become root",
+				       _("Don't become root"),
 				       get_color("Error")));
 	}
 	}
@@ -717,11 +803,43 @@ static void actually_do_package_run()
 
 void do_package_run_or_show_preview()
 {
+  // UI: check that something will actually be done first.
+  bool some_action_happening=false;
+  bool some_non_simple_keep_happening=false;
+  for(pkgCache::PkgIterator i=(*apt_cache_file)->PkgBegin(); !i.end(); ++i)
+    {
+      pkg_action_state state=find_pkg_state(i);
+
+      if(state!=pkg_unchanged)
+	{
+	  some_action_happening=true;
+
+	  if(!(state==pkg_hold && !(*apt_cache_file)->is_held(i)))
+	    some_non_simple_keep_happening=true;
+	}
+
+      if(some_action_happening && some_non_simple_keep_happening)
+	break;
+    }
+
+  if(!some_action_happening)
+    {
+      show_message(_("No packages are scheduled to be installed, removed, or upgraded."));
+      return;
+    }
+  else if(!some_non_simple_keep_happening)
+    {
+      show_message(_("No packages will be installed, removed or upgraded.\n\n"
+		     "Some packages could be upgraded, but you have not chosen to upgrade them.\n"
+		     "Type \"U\" to prepare an upgrade."));
+      return;
+    }
+
   if(apt_cache_file)
     {
       if(!active_preview_tree)
 	{
-	  if(aptcfg->FindB("Aptitude::Display-Planned-Action", true))
+	  if(aptcfg->FindB(PACKAGE "::Display-Planned-Action", true))
 	    do_show_preview();
 	  else
 	    actually_do_package_run();
@@ -759,7 +877,7 @@ void do_package_run()
 	actually_do_package_run();
       else if((*apt_cache_file)->BrokenCount()>0)
 	{
-	  if(aptcfg->FindB("Aptitude::Auto-Fix-Broken", true))
+	  if(aptcfg->FindB(PACKAGE "::Auto-Fix-Broken", true))
 	    fix_pkgs_and_do_package_run();
 	  else
 	    prompt_yesno(_("Some packages are broken.  Try to fix them?"),
@@ -796,9 +914,9 @@ void do_update_lists()
 	  popup_widget(vs_dialog_yesno(_("Updating the package lists requires administrative privileges, which\nyou currently do not have.  Would you like to change to the root account?\n"),
 				       bind(SigC::slot(&do_su_to_root),
 					    "-u"),
-				       "Become root",
+				       _("Become root"),
 				       SigC::slot(&really_do_update_lists),
-				       "Don't become root",
+				       _("Don't become root"),
 				       get_color("Error")));
 	}
     }
@@ -883,12 +1001,14 @@ static void do_mark_upgradable()
     {
       undo_group *undo=new undo_group;
 
-      (*apt_cache_file)->mark_all_upgradable(false, undo);
+      (*apt_cache_file)->mark_all_upgradable(true, undo);
 
       if(!undo->empty())
 	apt_undos->add_item(undo);
       else
 	delete undo;
+
+      package_states_changed();
     }
 }
 
@@ -900,13 +1020,15 @@ void do_forget_new()
       (*apt_cache_file)->forget_new(&undo);
       if(undo)
 	apt_undos->add_item(undo);
+
+      package_states_changed();
     }
 }
 
 static void do_undo()
 {
   apt_undos->undo();
-  vscreen_update();
+  package_states_changed();
 }
 
 #ifdef WITH_RELOAD_CACHE
@@ -957,7 +1079,7 @@ vs_menu_info actions_menu[]={
 	       N_("Delete package files which can no longer be downloaded"),
 	       SigC::slot(do_autoclean)),
 
-  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("Mark ^Upgradable"), NULL,
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("Mark ^Upgradable"), "MarkUpgradable",
 	       N_("Mark all upgradable packages which are not held for upgrade"),
 	       SigC::slot(do_mark_upgradable)),
 
@@ -1018,22 +1140,22 @@ vs_menu_info views_menu_info[]={
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Next"), "CycleNext",
 	       N_("View next display"), SigC::slot(do_view_next)),
 
-  vs_menu_info(vs_menu_info::VS_MENU_ITEM, "^Prev", "CyclePrev",
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Prev"), "CyclePrev",
 	       N_("View previous display"), SigC::slot(do_view_prev)),
 
-  vs_menu_info(vs_menu_info::VS_MENU_ITEM, "^Close", "Quit",
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Close"), "Quit",
 	       N_("Close this display"), SigC::slot(do_destroy_visible)),
 
   VS_MENU_SEPARATOR,
 
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("New Package ^View"), NULL,
 	       N_("Create a new default package view"),
-	       SigC::slot(do_new_package_view)),
+	       SigC::slot(do_new_package_view_with_new_bar)),
 
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("New Categorical ^Browser"),
 	       NULL,
 	       N_("Browse packages by category"),
-	       SigC::slot(do_new_hier_view)),
+	       SigC::slot(do_new_hier_view_with_new_bar)),
 
   VS_MENU_SEPARATOR,
   VS_MENU_END
@@ -1051,6 +1173,10 @@ vs_menu_info help_menu_info[]={
 	       N_("View the detailed program manual"),
 	       SigC::slot(do_help_readme)),
 
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^FAQ"), NULL,
+	       N_("View a list of frequently asked questions"),
+	       SigC::slot(do_help_faq)),
+
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^ChangeLog"), NULL,
 	       N_("View the important changes made in each version of the program"),
 	       SigC::slot(do_help_news)),
@@ -1122,6 +1248,10 @@ void ui_init()
   vscreen_init();
   init_defaults();
 
+  // The basic behavior of the package state signal is to update the
+  // display.
+  package_states_changed.connect(SigC::slot(vscreen_update));
+
   if(aptcfg->Exists(PACKAGE "::Theme"))
     load_options(PACKAGE "::Themes::"+string(aptcfg->Find(PACKAGE "::Theme"))+"::"+PACKAGE "::UI", true);
 
@@ -1132,9 +1262,9 @@ void ui_init()
 
   main_menu=new vs_menubar(!aptcfg->FindB(PACKAGE "::UI::Menubar-Autohide", false));
 
-  aptcfg->connectB(PACKAGE "::UI::Menubar-Autohide",
+  aptcfg->connectB(string(PACKAGE "::UI::Menubar-Autohide"),
 		   SigC::slot(do_set_menubar_autohide));
-  aptcfg->connect(PACKAGE "::UI::Package-Display-Format",
+  aptcfg->connect(string(PACKAGE "::UI::Package-Display-Format"),
 		  SigC::slot(do_setup_columns));
 
   cache_closed.connect(SigC::slot(do_show_reload_message));
@@ -1189,6 +1319,9 @@ void ui_init()
   main_stacked->connect_key_post("UpdatePackageList",
 				 &global_bindings,
 				 SigC::slot(do_update_lists));
+  main_stacked->connect_key_post("MarkUpgradable",
+				 &global_bindings,
+				 SigC::slot(do_mark_upgradable));
   main_stacked->connect_key_post("ForgetNewPackages",
 				 &global_bindings,
 				 SigC::slot(do_forget_new));
@@ -1211,7 +1344,7 @@ void ui_init()
     install_key=global_bindings.keyname("DoInstallRun");
 
   char buf[512];
-  snprintf(buf, 512, _("%s: Menu  %s: Help  %s: Quit  %s: Update  %s: Download/Install Pkgs"),
+  snprintf(buf, 512, _("%s: Menu  %s: Help  %s: Quit  %s: Update  %s: Download/Install/Remove Pkgs"),
 	   menu_key.c_str(),
 	   help_key.c_str(),
 	   quit_key.c_str(),
@@ -1222,8 +1355,8 @@ void ui_init()
   main_table->add_widget_opts(help_label, 0, 0, 1, 1,
 			      vs_table::EXPAND | vs_table::SHRINK,
 			      vs_table::ALIGN_CENTER);
-  aptcfg->connectB(PACKAGE "::UI::HelpBar",
-		   SigC::slot(help_label, &vscreen_widget::set_visible));
+  aptcfg->connectB(string(PACKAGE "::UI::HelpBar"),
+		   SigC::slot(*help_label, &vscreen_widget::set_visible));
 
   main_multiplex=new vs_multiplex;
   main_table->add_widget_opts(main_multiplex, 1, 0, 1, 1,
@@ -1294,12 +1427,23 @@ static void setup_main_widget(vscreen_widget *w, const std::string &menuref,
 
   // FIXME: if w is removed from the multiplexer but not destroyed, this may
   //       break.  Fix for now: Don't Do That Then!
-  w->destroyed.connect(bind(slot(views_menu, &vs_menu::remove_item), menuentry));
-  menuentry->selected.connect(slot(w, &vscreen_widget::show));
+  w->destroyed.connect(bind(slot(*views_menu, &vs_menu::remove_item), menuentry));
+  menuentry->selected.connect(slot(*w, &vscreen_widget::show));
 
   views_menu->append_item(menuentry);
 }
 
+// Handles the case where the last view is destroyed directly (other than
+// through do_destroy_visible); for instance, when a download completes.
+static void main_widget_destroyed()
+{
+  if(aptcfg->FindB(PACKAGE "::UI::Exit-On-Last-Close", true) &&
+     main_multiplex->num_children()==0)
+    // Don't prompt -- if the last view is destroyed, assume it was by
+    // the user's request.
+    file_quit();
+}
+
 void add_main_widget(vscreen_widget *w, const std::string &menuref,
 		     const std::string &menudesc)
 {
@@ -1311,6 +1455,7 @@ void add_main_widget(vscreen_widget *w, const std::string &menuref,
   setup_main_widget(w, menuref, menudesc);
   main_multiplex->add_widget(w);
   w->show();
+  w->destroyed.connect(SigC::slot(main_widget_destroyed));
 }
 
 void insert_main_widget(vscreen_widget *w, const std::string &menuref,
@@ -1335,12 +1480,21 @@ vs_progress *gen_progress_bar()
   return rval;
 }
 
+fragment *wrapbox(fragment *contents)
+{
+  if(aptcfg->FindB(PACKAGE "::UI::Fill-Text", false))
+    return fillbox(contents);
+  else
+    return flowbox(contents);
+}
+
 static void reset_status_download()
 {
   active_status_download=NULL;
 }
 
 download_manager *gen_download_progress(bool force_noninvasive,
+					const string &title,
 					Slot0Arg abortslot)
 {
   download_manager *m=new download_manager;
@@ -1357,18 +1511,18 @@ download_manager *gen_download_progress(bool force_noninvasive,
   else
     {
       w=new download_list(abortslot);
-      add_main_widget(w, _("Updating package lists"), "");
+      add_main_widget(w, title, "");
     }
 
-  m->MediaChange_sig.connect(slot(w, &download_list::MediaChange));
-  m->IMSHit_sig.connect(slot(w, &download_list::IMSHit));
-  m->Fetch_sig.connect(slot(w, &download_list::Fetch));
-  m->Done_sig.connect(slot(w, &download_list::Done));
-  m->Fail_sig.connect(slot(w, &download_list::Fail));
-  m->Pulse_sig.connect(slot(w, &download_list::Pulse));
-  m->Start_sig.connect(slot(w, &download_list::Start));
-  m->Stop_sig.connect(slot(w, &download_list::Stop));
-  m->Complete_sig.connect(slot(w, &download_list::Complete));
+  m->MediaChange_sig.connect(slot(*w, &download_list::MediaChange));
+  m->IMSHit_sig.connect(slot(*w, &download_list::IMSHit));
+  m->Fetch_sig.connect(slot(*w, &download_list::Fetch));
+  m->Done_sig.connect(slot(*w, &download_list::Done));
+  m->Fail_sig.connect(slot(*w, &download_list::Fail));
+  m->Pulse_sig.connect(slot(*w, &download_list::Pulse));
+  m->Start_sig.connect(slot(*w, &download_list::Start));
+  m->Stop_sig.connect(slot(*w, &download_list::Stop));
+  m->Complete_sig.connect(slot(*w, &download_list::Complete));
 
   return m;
 }
@@ -1378,7 +1532,7 @@ static void do_prompt_string(string s,
 			     SigC::Slot0<void> realslot)
 {
   e->add_to_history(s);
-  realslot.call();
+  realslot();
 }
 
 void prompt_string(const std::string &prompt,
@@ -1396,13 +1550,13 @@ void prompt_string(const std::string &prompt,
 
       e->entered.connect(bind(SigC::slot(do_prompt_string),
 			      e,
-			      SigC::slot(e, &vscreen_widget::destroy)));
+			      SigC::slot(*e, &vscreen_widget::destroy)));
       if(changed_slot)
 	e->text_changed.connect(changed_slot);
 
       e->connect_key("Cancel",
 		     &global_bindings,
-		     SigC::slot(e, &vscreen_widget::destroy));
+		     SigC::slot(*e, &vscreen_widget::destroy));
 
       if(cancel_slot)
 	e->connect_key("Cancel",
@@ -1435,12 +1589,12 @@ static void do_prompt_yesno(int cval,
   if(rval)
     {
       if(yesslot)
-	((SigC::Slot0<void>) yesslot).call();
+	((SigC::Slot0<void>) yesslot)();
     }
   else
     {
       if(noslot)
-	((SigC::Slot0<void>) noslot).call();
+	((SigC::Slot0<void>) noslot)();
     }
 }
 
