@@ -11,7 +11,7 @@
 
 #include <apt-pkg/pkgcache.h>
 
-#include <sigc++/basic_signal.h>
+#include <sigc++/signal.h>
 
 #include <string>
 
@@ -19,10 +19,13 @@
 #include <vscreen/vs_editline.h>
 // Bleah..this needs to be included so histories work
 
-class vscreen_widget;
-class vs_progress;
+class OpProgress;
+
 class download_manager;
+class fragment;
 class pkg_hier;
+class vs_progress;
+class vscreen_widget;
 
 /******************************************************************************
  * Global signals:
@@ -31,6 +34,13 @@ class pkg_hier;
 // File menu
 extern SigC::Signal0<void> file_quit;
 
+// A signal that widgets watching package states should update themselves.
+//
+// This should be triggered at the end of any keystroke (or mouse-stroke)
+// handler that alters the package states.  At a minimum, it will
+// call vscreen_update().
+extern SigC::Signal0<void> package_states_changed;
+
 /*****************************************************************************/
 
 void ui_init();
@@ -53,16 +63,24 @@ void insert_main_widget(vscreen_widget *w, const std::string &menuref,
 // Generates a progress bar.
 vs_progress *gen_progress_bar();
 
-// Similarly for download progress displays.
-//
-// force_noninvasive tells whether to use a 'non-invasive' display (ie, one
-// that doesn't take over the screen..for  instance, the "status-line"-style
-// download bar)
-//
-// abortslot, if present, is a slot which will abort the operation if invoked.
-// (eg, so you can download packages but not install them)
-download_manager *gen_download_progress(bool force_noninvasive=false,
-					Slot0Arg abortslot=NULL);
+// Generates an appropriate box to wrap text.
+fragment *wrapbox(fragment *contents);
+
+/** Generates a download progress object based on the user's current
+ *  settings.
+ *
+ *  \param force_noninvasive if \b true, the generated UI object will
+ *            always be "noninvasive" (typically a bar at the bottom of
+ *            the screen).
+ *  \param title             if a new view is generated, this string is
+ *                           used as its title.
+ *  \param abortslot         the slot to trigger if the download is aborted.
+ *
+ *  \return the new download manager.
+ */
+download_manager *gen_download_progress(bool force_noninvasive,
+					const std::string &title,
+					Slot0Arg abortslot);
 
 // Asks the user for simple input (the question will appear in a "minibuffer"
 // or in a dialog according to preferences)
@@ -89,7 +107,7 @@ void show_message(const std::string &msg,
 // Can be used to manually pop up an error dialog, if necessary.
 void check_apt_errors();
 
-void do_new_package_view();
+void do_new_package_view(OpProgress &progress);
 // Displays a new package-view.
 
 void do_package_run_or_show_preview();
