@@ -25,6 +25,83 @@
 #include <string>
 #include <curses.h>
 
+#include <assert.h>
+
+// For isspace
+#include <ctype.h>
+
+#include "../../config.h"
+
+/** Based on libstdc++-3's instantiation of this for characters.
+ *
+ *  This could do something clever, such as changing comparisons to
+ *  only apply to the text itself, but it doesn't; I'm not sure what
+ *  the effect of changes like that would be.
+ *
+ *  The int_type stuff is not implemented; it could perhaps be
+ *  implemented later if it's needed.
+ */
+namespace std {
+struct TRAITS_CLASS<chtype> {
+  typedef chtype char_type;
+
+  static void assign (char_type& c1, const char_type& c2)
+    { c1 = c2; }
+  static bool eq (const char_type & c1, const char_type& c2)
+    { return (c1 == c2); }
+  static bool ne (const char_type& c1, const char_type& c2)
+    { return (c1 != c2); }
+  static bool lt (const char_type& c1, const char_type& c2)
+    { return (c1 < c2); }
+  static char_type eos () { return 0; }
+  static bool is_del(char_type a) { return isspace(a|A_CHARTEXT); }
+
+  static int compare (const char_type* s1, const char_type* s2, size_t n);
+  static size_t length (const char_type* s);
+  static char_type* copy (char_type* s1, const char_type* s2, size_t n)
+    { return (char_type*) memcpy (s1, s2, n*sizeof(char_type)); }
+  static char_type* move (char_type* s1, const char_type* s2, size_t n)
+    { return (char_type*) memmove (s1, s2, n*sizeof(char_type)); }
+  static char_type* assign (char_type* s1, size_t n, const char_type& c);
+};
+}
+
+/** A string class which stores attributes along with characters.
+ *
+ *  This has to derive from basic_string in order to implement operator=
+ *  (which can't be a global function -- it has to be a nonstatic member)
+ */
+class chstring:public std::basic_string<chtype>
+{
+  typedef std::basic_string<chtype> super;
+public:
+  chstring(const std::basic_string<chtype> &s)
+    :std::basic_string<chtype>(s) {}
+
+  chstring(const std::string &s);
+  chstring(const std::string &s, int attr);
+
+  chstring(const chstring &s):super(s) {}
+
+  chstring(const chstring &s, size_t loc, size_t n=npos)
+    :super(s, loc, n) {}
+
+  chstring(size_t n, chtype c)
+    :super(n, c) {}
+
+  /** Assign the characters of s to this, setting all attributes to A_NORMAL. */
+  chstring &operator=(const std::string &s);
+
+  /** Add attrs to each character of this chstring. */
+  void add_attr(int attrs);
+
+  /** Remove attrs from each character of this chstring. */
+  void remove_attr(int attrs);
+
+  /** Set the attributes of each character of this chstring to attrs. */
+  void set_attr(int attrs);
+};
+
 inline chtype _getbkgd(WINDOW *win)
 {
   return getbkgd(win);
@@ -247,6 +324,11 @@ public:
   int mvaddstr(int y, int x, const char *str) {return mvwaddstr(win, y, x, str);}
   int mvaddnstr(int y, int x, const char *str, int n) {return mvwaddnstr(win, y, x, str, n);}
 
+  int addstr(const chstring &str) {return waddchstr(win, str.c_str());}
+  int addnstr(const chstring &str, int n) {return waddchnstr(win, str.c_str(), n);}
+  int mvaddstr(int y, int x, const chstring &str) {return mvwaddchstr(win, y, x, str.c_str());}
+  int mvaddnstr(int y, int x, const chstring &str, int n) {return mvwaddchnstr(win, y, x, str.c_str(), n);}
+
   int attroff(int attrs) {return wattroff(win, attrs);}
   int attron(int attrs) {return wattron(win, attrs);}
   int attrset(int attrs) {return wattrset(win, attrs);}
