@@ -7,6 +7,7 @@
 #include "config/colors.h"
 
 #include <sigc++/bind.h>
+#include <sigc++/object_slot.h>
 
 using namespace std;
 
@@ -15,10 +16,10 @@ keybindings *vs_menubar::bindings=NULL;
 vs_menubar::vs_menubar(bool _always_visible)
   :vs_container(), active(false), always_visible(_always_visible), curloc(0), subwidget(NULL)
 {
-  do_layout.connect(slot(this, &vs_menubar::layout_me));
+  do_layout.connect(slot(*this, &vs_menubar::layout_me));
 
-  focussed.connect(slot(this, &vs_menubar::got_focus));
-  unfocussed.connect(slot(this, &vs_menubar::lost_focus));
+  focussed.connect(slot(*this, &vs_menubar::got_focus));
+  unfocussed.connect(slot(*this, &vs_menubar::lost_focus));
 }
 
 vs_menubar::~vs_menubar()
@@ -99,9 +100,9 @@ void vs_menubar::append_item(string title, vs_menu *menu)
 {
   items.push_back(item(' '+title+' ', menu));
 
-  menu->shown_sig.connect(bind(slot(this, &vs_menubar::show_menu), menu));
-  menu->hidden_sig.connect(bind(slot(this, &vs_menubar::hide_menu), menu));
-  menu->menus_goaway.connect(slot(this, &vs_menubar::disappear));
+  menu->shown_sig.connect(bind(slot(*this, &vs_menubar::show_menu), menu));
+  menu->hidden_sig.connect(bind(slot(*this, &vs_menubar::hide_menu), menu));
+  menu->menus_goaway.connect(slot(*this, &vs_menubar::disappear));
   menu->set_owner(this);
 
   vscreen_update();
