@@ -9,18 +9,40 @@
 #include "config/keybindings.h"
 
 #include <sigc++/bind.h>
+#include <sigc++/object_slot.h>
 
 using namespace std;
 
 keybindings *vs_table::bindings=NULL;
 
 vs_table::vs_table()
+  :rowsep(0), colsep(0)
 {
-  do_layout.connect(slot(this, &vs_table::layout_me));
+  do_layout.connect(slot(*this, &vs_table::layout_me));
   focus=children.end();
 
-  focussed.connect(slot(this, &vs_table::got_focus));
-  unfocussed.connect(slot(this, &vs_table::lost_focus));
+  focussed.connect(slot(*this, &vs_table::got_focus));
+  unfocussed.connect(slot(*this, &vs_table::lost_focus));
+}
+
+void vs_table::set_rowsep(int n)
+{
+  if(n!=rowsep)
+    {
+      rowsep=n;
+      if(get_visible())
+	vscreen_update();
+    }
+}
+
+void vs_table::set_colsep(int n)
+{
+  if(n!=colsep)
+    {
+      colsep=n;
+      if(get_visible())
+	vscreen_update();
+    }
 }
 
 // We need to call get_focus() here to update the "focus" pointer.
@@ -81,8 +103,8 @@ void vs_table::add_widget_opts(vscreen_widget *w, int row_start, int col_start,
       // FIXME: throw something/print a nasty error message?
       abort();
 
-  SigC::Connection shown_conn=w->shown_sig.connect(bind(slot(this, &vs_table::show_widget), w));
-  SigC::Connection hidden_conn=w->hidden_sig.connect(bind(slot(this, &vs_table::hide_widget), w));
+  SigC::Connection shown_conn=w->shown_sig.connect(bind(slot(*this, &vs_table::show_widget), w));
+  SigC::Connection hidden_conn=w->hidden_sig.connect(bind(slot(*this, &vs_table::hide_widget), w));
 
   children.push_back(child_info(w, row_start, col_start, row_span, col_span, xopts, yopts, shown_conn, hidden_conn));
   w->set_owner(this);
@@ -417,8 +439,7 @@ bool vs_table::handle_char(chtype ch)
 	return true;
       else if(bindings->key_matches(ch, "Cycle"))
 	{
-	  if(get_isfocussed())
-	    focus->w->unfocussed();
+	  childlist::iterator oldfocus=focus;
 
 	  focus++;
 
@@ -432,78 +453,89 @@ bool vs_table::handle_char(chtype ch)
 		focus++;
 	    }
 
-	  if(focus!=children.end())
+	  if(focus!=children.end() && focus!=oldfocus)
 	    {
 	      if(get_isfocussed())
-		focus->w->focussed();
+		{
+		  oldfocus->w->unfocussed();
+		  focus->w->focussed();
+		}
 	      vscreen_updatecursor();
 	    }
 
-	  return true;
+	  return focus!=oldfocus;
 	}
       else if(bindings->key_matches(ch, "Left"))
 	{
-	  if(get_isfocussed())
-	    focus->w->unfocussed();
+	  childlist::iterator oldfocus=focus;
 
 	  focus=find_best_focus(focus, -1, 0);
 
-	  if(focus!=children.end())
+	  if(focus!=children.end() && focus!=oldfocus)
 	    {
 	      if(get_isfocussed())
-		focus->w->focussed();
+		{
+		  oldfocus->w->unfocussed();
+		  focus->w->focussed();
+		}
 	      vscreen_updatecursor();
 	    }
 
-	  return true;
+	  return focus!=oldfocus;
 	}
       else if(bindings->key_matches(ch, "Right"))
 	{
-	  if(get_isfocussed())
-	    focus->w->unfocussed();
+	  childlist::iterator oldfocus=focus;
 
 	  focus=find_best_focus(focus, 1, 0);
 
-	  if(focus!=children.end())
+	  if(focus!=children.end() && focus!=oldfocus)
 	    {
 	      if(get_isfocussed())
-		focus->w->focussed();
+		{
+		  oldfocus->w->unfocussed();
+		  focus->w->focussed();
+		}
 	      vscreen_updatecursor();
 	    }
 
-	  return true;
+	  return focus!=oldfocus;
 	}
       else if(bindings->key_matches(ch, "Up"))
 	{
-	  if(get_isfocussed())
-	    focus->w->unfocussed();
+	  childlist::iterator oldfocus=focus;
 
 	  focus=find_best_focus(focus, 0, -1);
 
-	  if(focus!=children.end())
+	  if(focus!=children.end() && focus!=oldfocus)
 	    {
 	      if(get_isfocussed())
-		focus->w->focussed();
+		{
+		  oldfocus->w->unfocussed();
+		  focus->w->focussed();
+		}
 	      vscreen_updatecursor();
 	    }
 
-	  return true;
+	  return focus!=oldfocus;
 	}
       else if(bindings->key_matches(ch, "Down"))
 	{
-	  if(get_isfocussed())
-	    focus->w->unfocussed();
+	  childlist::iterator oldfocus=focus;
 
 	  focus=find_best_focus(focus, 0, 1);
 
-	  if(focus!=children.end())
+	  if(focus!=children.end() && focus!=oldfocus)
 	    {
 	      if(get_isfocussed())
-		focus->w->focussed();
+		{
+		  oldfocus->w->unfocussed();
+		  focus->w->focussed();
+		}
 	      vscreen_updatecursor();
 	    }
 
-	  return true;
+	  return focus!=oldfocus;
 	}
       else
 	return vs_passthrough::handle_char(ch);
@@ -693,6 +725,9 @@ void vs_table::calc_initial_sizes(int *row_sizes, int *col_sizes, int nrows, int
     final_size->h+=row_sizes[i];
   for(int j=0; j<ncols; j++)
     final_size->w+=col_sizes[j];
+
+  delete[] row_expandable;
+  delete[] col_expandable;
 }
 
 size vs_table::size_request()
@@ -733,16 +768,18 @@ void vs_table::layout_me()
       int nrows=num_rows(), ncols=num_cols();
       int *row_sizes=new int[nrows], *col_sizes=new int[ncols];
       size final_size(0,0);
+      int width=getmaxx()-colsep*max(ncols-1, 0);
+      int height=getmaxy()-rowsep*max(nrows-1, 0);
 
       calc_initial_sizes(row_sizes, col_sizes, nrows, ncols, &final_size);
 
-      if(final_size.h<getmaxy())
+      if(final_size.h<height)
 	{
 	  // We were allocated more space than we requested.
 	  // Find out how many rows can be expanded
 	  bool *can_expand=new bool[nrows];
 	  int n_expandable=0;
-	  int overflow=getmaxy()-final_size.h;
+	  int overflow=height-final_size.h;
 
 	  for(int i=0; i<nrows; i++)
 	    can_expand[i]=false;
@@ -792,7 +829,7 @@ void vs_table::layout_me()
 
 	  delete[] can_expand;
 	}
-      else if(final_size.h>getmaxy())
+      else if(final_size.h>height)
 	{
 	  // We weren't allocated enough space :(
 	  // Try to shrink as much as possible.  (never shrink anything below
@@ -800,7 +837,7 @@ void vs_table::layout_me()
 	  // row/column, it'll already be size 0 anyway)
 	  bool *can_shrink=new bool[nrows];
 	  int n_shrinkable=0;
-	  int shortfall=final_size.h-getmaxy();
+	  int shortfall=final_size.h-height;
 
 	  // To save time, don't even bother trying to shrink unshrinkable
 	  // rows..
@@ -858,12 +895,12 @@ void vs_table::layout_me()
 	      int cursize=0;
 
 	      int i=0;
-	      while(i<nrows && cursize+row_sizes[i]<getmaxy())
+	      while(i<nrows && cursize+row_sizes[i]<height)
 		cursize+=row_sizes[i++];
 
 	      assert(i<nrows);
 
-	      row_sizes[i++]=getmaxy()-cursize;
+	      row_sizes[i++]=height-cursize;
 
 	      while(i<nrows)
 		row_sizes[i++]=0;
@@ -873,13 +910,13 @@ void vs_table::layout_me()
 	}
 
       // Perform exactly the same calculations on columns
-      if(final_size.w<getmaxx())
+      if(final_size.w<width)
 	{
 	  // We were allocated more space than we requested.
 	  // Find out how many columns can be expanded
 	  bool *can_expand=new bool[ncols];
 	  int n_expandable=0;
-	  int overflow=getmaxx()-final_size.w;
+	  int overflow=width-final_size.w;
 
 	  for(int i=0; i<ncols; i++)
 	    can_expand[i]=false;
@@ -927,7 +964,7 @@ void vs_table::layout_me()
 
 	  delete[] can_expand;
 	}
-      else if(final_size.w>getmaxx())
+      else if(final_size.w>width)
 	{
 	  // We weren't allocated enough space :(
 	  // Try to shrink as much as possible.  (never shrink anything below
@@ -935,7 +972,7 @@ void vs_table::layout_me()
 	  // row/column, it'll already be size 0 anyway)
 	  bool *can_shrink=new bool[ncols];
 	  int n_shrinkable=0;
-	  int shortfall=final_size.w-getmaxx();
+	  int shortfall=final_size.w-width;
 
 	  // To save time, don't even bother trying to shrink unshrinkable
 	  // rows..
@@ -993,12 +1030,12 @@ void vs_table::layout_me()
 	      int cursize=0;
 
 	      int i=0;
-	      while(i<ncols && cursize+col_sizes[i]<getmaxx())
+	      while(i<ncols && cursize+col_sizes[i]<width)
 		cursize+=col_sizes[i++];
 
 	      assert(i<ncols);
 
-	      col_sizes[i++]=getmaxx()-cursize;
+	      col_sizes[i++]=width-cursize;
 
 	      while(i<ncols)
 		col_sizes[i++]=0;
@@ -1016,17 +1053,21 @@ void vs_table::layout_me()
 	    int w=0, h=0;
 
 	    for(int j=0; j<i->row_start; j++)
-	      y+=row_sizes[j];
+	      y+=row_sizes[j]+rowsep;
 
 	    for(int j=i->row_start; j<i->row_start+i->row_span; j++)
 	      h+=row_sizes[j];
 
+	    h+=rowsep*max(i->row_span-1, 0);
+
 	    for(int j=0; j<i->col_start; j++)
-	      x+=col_sizes[j];
+	      x+=col_sizes[j]+colsep;
 
 	    for(int j=i->col_start; j<i->col_start+i->col_span; j++)
 	      w+=col_sizes[j];
 
+	    w+=colsep*max(i->col_span-1, 0);
+
 	    int curx=x, cury=y, curw=w, curh=h;
 	    if(curw>i->request_size.w && !(i->x_options & EXPAND))
 	      {
