@@ -40,6 +40,9 @@
 #include "config/keybindings.h"
 #include "config/colors.h"
 
+// For _()
+#include "../aptitude.h"
+
 #include <signal.h>
 
 #ifdef HAVE_LIBPTHREAD
@@ -93,15 +96,16 @@ int ntimeouts=0;
 static vscreen_widget *toplevel=NULL;
 // The widget which is displayed as the root of everything
 
-static bool resized=false;
+//static bool resized=false;
 static void sigresize(int sig)
 {
-  resized=true;
+  vscreen_handleresize();
+  //resized=true;
 }
 
 // Cleanly shutdown (eg, restore screen settings if possible)
 //
-// Called on SIGTERM, SIGSEGV, SIGABRT, and SIGQUIT
+// Called on SIGTERM, SIGINT, SIGSEGV, SIGABRT, and SIGQUIT
 //
 // FIXME: revert to the /previous/ handler, not just SIG_DFL?
 static void sigkilled(int sig)
@@ -111,16 +115,16 @@ static void sigkilled(int sig)
   switch(sig)
     {
     case SIGTERM:
-      fprintf(stderr, "Ouch!  Got SIGTERM, dying..\n");
+      fprintf(stderr, _("Ouch!  Got SIGTERM, dying..\n"));
       break;
     case SIGSEGV:
-      fprintf(stderr, "Ouch!  Got SIGSEGV, dying..\n");
+      fprintf(stderr, _("Ouch!  Got SIGSEGV, dying..\n"));
       break;
     case SIGABRT:
-      fprintf(stderr, "Ouch!  Got SIGABRT, dying..\n");
+      fprintf(stderr, _("Ouch!  Got SIGABRT, dying..\n"));
       break;
     case SIGQUIT:
-      fprintf(stderr, "Ouch!  Got SIGQUIT, dying..\n");
+      fprintf(stderr, _("Ouch!  Got SIGQUIT, dying..\n"));
       break;
     }
 
@@ -313,7 +317,13 @@ void vscreen_init()
   if(toplevel)
     vscreen_settoplevel(toplevel);
 
+  vscreen_install_sighandlers();
+}
+
+void vscreen_install_sighandlers()
+{
   signal(SIGTERM, sigkilled);
+  signal(SIGINT, sigkilled);
   signal(SIGSEGV, sigkilled);
   signal(SIGQUIT, sigkilled);
   signal(SIGABRT, sigkilled);
@@ -461,9 +471,14 @@ bool vscreen_poll()
 
   if(toplevel)
     {
-      toplevel->win.nodelay(true);
-      chtype ch=toplevel->win.getch();
-      toplevel->win.nodelay(false);
+      chtype ch;
+
+      do
+	{
+	  toplevel->win.nodelay(true);
+	  ch=toplevel->win.getch();
+	  toplevel->win.nodelay(false);
+	} while(ch==(chtype) KEY_RESIZE);
 
       sigprocmask(SIG_BLOCK, &signals, &prevsignals);
 
@@ -538,15 +553,21 @@ void vscreen_mainloop(int synch)
       // location; I just want to be sure that I reset the timeout of the
       // correct widget. (??)
 
-      vscreen_releaselock();
-      chtype ch=curwidget->get_win().getch();
-      vscreen_acquirelock();
+      chtype ch;
 
-      if(resized)
+      do
 	{
-	  resized=false;
-	  vscreen_handleresize();
-	}
+	  vscreen_releaselock();
+	  ch=curwidget->get_win().getch();
+	  vscreen_acquirelock();
+	} while(ch==KEY_RESIZE);
+
+      // used when asynchronous resizing was dangerous
+      //if(resized)
+      //	{
+      //  resized=false;
+      //  vscreen_handleresize();
+      //}
 
       curwidget->timeout(prevtimeout);
 
